# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


import unittest
import tempfile
import shutil
from knack.util import CLIError
from unittest import mock

from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives import serialization

from azure.cli.core.keys import generate_ssh_keys


class TestGenerateSSHKeys(unittest.TestCase):

    def setUp(self):
        # set up temporary directory to be used for temp files.
        self._tempdirName = tempfile.mkdtemp(prefix="key_tmp_")

        self.key = rsa.generate_private_key(public_exponent=65537, key_size=2048)
        self.private_key = self.key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.TraditionalOpenSSL,
            encryption_algorithm=serialization.NoEncryption()
        ).decode()
        self.public_key = self.key.public_key().public_bytes(
            encoding=serialization.Encoding.OpenSSH,
            format=serialization.PublicFormat.OpenSSH
        ).decode()

    def tearDown(self):
        # delete temporary directory to be used for temp files.
        shutil.rmtree(self._tempdirName)

    def test_when_public_key_file_exists(self):
        # Create public key file
        public_key_path = self._create_new_temp_key_file(self.public_key, suffix=".pub")

        # Create private key file
        private_key_path = self._create_new_temp_key_file(self.private_key)

        # Call generate_ssh_keys and assert that returned public key same as original
        new_public_key = generate_ssh_keys("", public_key_path)
        self.assertEqual(self.public_key, new_public_key)

        # Check that private and public key file contents unchanged.
        with open(public_key_path, 'r') as f:
            new_public_key = f.read()
            self.assertEqual(self.public_key, new_public_key)

        with open(private_key_path, 'r') as f:
            new_private_key = f.read()
            self.assertEqual(self.private_key, new_private_key)

    def test_error_raised_when_public_key_file_exists_IOError(self):
        # Create public key file
        public_key_path = self._create_new_temp_key_file(self.public_key)

        with mock.patch('azure.cli.core.keys.open') as mocked_open:
            # mock failed call to read
            mocked_f = mocked_open.return_value.__enter__.return_value
            mocked_f.read = mock.MagicMock(side_effect=OSError("Mocked IOError"))

            # assert that CLIError raised when generate_ssh_keys is called
            with self.assertRaises(CLIError):
                generate_ssh_keys("", public_key_path)

            # assert that CLIError raised because of attempt to read public key file.
            mocked_open.assert_called_once_with(public_key_path, 'r')
            mocked_f.read.assert_called_once()

    def test_error_raised_when_private_key_file_exists_encrypted(self):
        # Create empty private key file
        private_key_path = self._create_new_temp_key_file("")

        # Write encrypted / passworded key into file
        private_bytes = self.key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.TraditionalOpenSSL,
            encryption_algorithm=serialization.BestAvailableEncryption(b'test')
        )
        with open(private_key_path, 'wb') as f:
            f.write(private_bytes)

        # generate_ssh_keys should raise
        #   TypeError: Password was not given but private key is encrypted
        with self.assertRaises(TypeError):
            public_key_path = private_key_path + ".pub"
            generate_ssh_keys(private_key_path, public_key_path)

    def test_generate_public_key_file_from_existing_private_key_files(self):
        # Create private key file
        private_key_path = self._create_new_temp_key_file(self.private_key)

        # Call generate_ssh_keys and assert that returned public key same as original
        public_key_path = private_key_path + ".pub"
        new_public_key = generate_ssh_keys(private_key_path, public_key_path)
        self.assertEqual(self.public_key, new_public_key)

        # Check that correct public key file has been created
        with open(public_key_path, 'r') as f:
            public_key = f.read()
            self.assertEqual(self.public_key, public_key)

        # Check that private key file contents unchanged
        with open(private_key_path, 'r') as f:
            private_key = f.read()
            self.assertEqual(self.private_key, private_key)

    def test_generate_new_private_public_key_files(self):
        # create random temp file name
        f = tempfile.NamedTemporaryFile(mode='w', dir=self._tempdirName)
        f.close()
        private_key_path = f.name

        # Call generate_ssh_keys and assert that returned public key same as original
        public_key_path = private_key_path + ".pub"
        new_public_key = generate_ssh_keys(private_key_path, public_key_path)

        # Check that public key returned is same as public key in public key path
        with open(public_key_path, 'r') as f:
            public_key = f.read()
            self.assertEqual(public_key, new_public_key)

        # Check that public key corresponds to private key
        with open(private_key_path, 'rb') as f:
            private_bytes = f.read()

        private_key = serialization.load_pem_private_key(private_bytes, password=None)
        public_key = private_key.public_key().public_bytes(
            encoding=serialization.Encoding.OpenSSH,
            format=serialization.PublicFormat.OpenSSH
        ).decode()
        self.assertEqual(public_key, new_public_key)

    def _create_new_temp_key_file(self, key_data, suffix=""):
        with tempfile.NamedTemporaryFile(mode='w', dir=self._tempdirName, delete=False, suffix=suffix) as f:
            f.write(key_data)
            return f.name


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer


class CognitiveServicesPropertiesTests(ScenarioTest):
    @ResourceGroupPreparer()
    def test_cognitiveservices_account_capabilities(self, resource_group):
        sname = self.create_random_name(prefix='cs_cli_test_', length=16)

        self.kwargs.update({
            'sname': sname,
            'kind': 'FormRecognizer',
            'sku': 'S0',
            'location': 'SOUTHCENTRALUS'
        })

        # test to create cognitive services account
        self.cmd('az cognitiveservices account create -n {sname} -g {rg} --kind {kind} --sku {sku} -l {location} --yes',
                 checks=[self.check('name', '{sname}'),
                         self.check('location', '{location}'),
                         self.check('sku.name', '{sku}')])
        account = self.cmd('az cognitiveservices account show -n {sname} -g {rg}').get_output_in_json()
        self.assertTrue(account['properties']['capabilities'] is not None)
        self.assertTrue(len(account['properties']['capabilities']) > 0)
        self.assertTrue(account['properties']['capabilities'][0]['name'] is not None)
        self.assertTrue(account['properties']['capabilities'][0]['name'] != '')

        # delete the cognitive services account
        ret = self.cmd('az cognitiveservices account delete -n {sname} -g {rg}')
        self.assertEqual(ret.exit_code, 0)

    @ResourceGroupPreparer()
    def test_cognitiveservices_account_public_network_access(self, resource_group):
        sname = self.create_random_name(prefix='cs_cli_test_', length=16)

        self.kwargs.update({
            'sname': sname,
            'kind': 'Face',
            'sku': 'S0',
            'location': 'SOUTHCENTRALUS'
        })

        # test to create cognitive services account
        self.cmd('az cognitiveservices account create -n {sname} -g {rg} --kind {kind} --sku {sku} -l {location} --yes',
                 checks=[self.check('name', '{sname}'),
                         self.check('location', '{location}'),
                         self.check('sku.name', '{sku}')])
        account = self.cmd('az cognitiveservices account show -n {sname} -g {rg}').get_output_in_json()
        print(account)
        self.assertEqual(account['properties']['publicNetworkAccess'], 'Enabled')


        # delete the cognitive services account
        ret = self.cmd('az cognitiveservices account delete -n {sname} -g {rg}')
        self.assertEqual(ret.exit_code, 0)

    @ResourceGroupPreparer()
    def test_cognitiveservices_aiservices(self, resource_group):
        sname = self.create_random_name(prefix='csclitest_', length=16)
        sname2 = self.create_random_name(prefix='csclitest2_', length=16)
        self.kwargs.update({
            'sname': sname,
            'sname2': sname2,
            'kind': 'AIServices',
            'sku': 'S0',
            'location': 'EASTUS'
        })

        # test to create cognitive services account with allow_project_management set to True by default for AIServices kind
        self.cmd('az cognitiveservices account create -n {sname} -g {rg} --kind {kind} --sku {sku} -l {location} --yes',
                 checks=[self.check('name', '{sname}'),
                         self.check('location', '{location}'),
                         self.check('sku.name', '{sku}'),
                         self.check('properties.allowProjectManagement', True)])
        account = self.cmd('az cognitiveservices account show -n {sname} -g {rg}').get_output_in_json()
        self.assertTrue(account['identity']['type'] == 'SystemAssigned')

        # delete the cognitive services account
        ret = self.cmd('az cognitiveservices account delete -n {sname} -g {rg}')
        self.assertEqual(ret.exit_code, 0)

        self.cmd('az cognitiveservices account create -n {sname2} -g {rg} --kind {kind} --sku {sku} -l {location} --allow-project-management true --yes',
                 checks=[self.check('name', '{sname2}'),
                         self.check('location', '{location}'),
                         self.check('sku.name', '{sku}'),
                         self.check('properties.allowProjectManagement', True)])
        account = self.cmd('az cognitiveservices account show -n {sname2} -g {rg}').get_output_in_json()
        self.assertTrue(account['identity']['type'] == 'SystemAssigned')

        # delete the cognitive services account
        ret = self.cmd('az cognitiveservices account delete -n {sname2} -g {rg}')
        self.assertEqual(ret.exit_code, 0)


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long

import json
from azure.cli.core import decorators

sensitive_data_warning_message = '[Warning] This output may compromise security by showing secrets. Learn more at: https://go.microsoft.com/fwlink/?linkid=2258669'
sensitive_data_detailed_warning_message = '[Warning] This output may compromise security by showing the following secrets: {}. Learn more at: https://go.microsoft.com/fwlink/?linkid=2258669'


@decorators.call_once
def get_secret_masker():
    # global secret_masker_instance
    from microsoft_security_utilities_secret_masker import SecretMasker, load_regex_patterns_from_json_file
    regex_patterns = load_regex_patterns_from_json_file('HighConfidenceSecurityModels.json')
    return SecretMasker(regex_patterns)


def is_containing_credential(content, is_file=False):
    """Check if the given content contains credential or not.

    :param content: The content or the file path.
    :type content: Any.
    :param is_file: flag to clarify the content is file path or not.
    :type is_file: bool.
    :param max_level: the max level of credential to check.
    :type max_level: int.
    :returns:  bool.
    """
    if is_file:
        with open(content, 'r') as f:
            content = f.read()
    elif not isinstance(content, str):
        try:
            content = json.dumps(content)
        except TypeError:
            content = str(content)
        except ValueError:
            raise ValueError('The content is not string or json object.')
    return get_secret_masker().detect_secrets(content)


def distinguish_credential(content, is_file=False):
    """Distinguish which property contains credential from the given content.

    :param content: The content(can be string or json object) or the file path.
    :type content: Any.
    :param is_file: flag to clarify the content is file path or not.
    :type is_file: bool.
    :param max_level: the max level of credential to check.
    :type max_level: int.
    :returns: bool, set.
    """
    containing_credential = False
    secret_property_names = set()
    secret_names = set()
    if is_file:
        with open(content, 'r') as f:
            content = json.load(f)

    if isinstance(content, list):
        for item in content:
            _containing_credential, _secret_property_names, _secret_names = distinguish_credential(item)
            containing_credential = containing_credential or _containing_credential
            secret_property_names.update(_secret_property_names)
            secret_names.update(_secret_names)
        return containing_credential, secret_property_names, secret_names

    if isinstance(content, dict):
        for key, value in content.items():
            _containing_credential, _secret_property_names, _secret_names = distinguish_credential(value)
            containing_credential = containing_credential or _containing_credential
            secret_property_names.update(_secret_property_names)
            if _containing_credential:
                secret_property_names.add(key)
                secret_names.update(_secret_names)
        return containing_credential, secret_property_names, secret_names

    detections = is_containing_credential(content)
    if detections:
        containing_credential = True
        for detection in detections:
            secret_names.add(detection.name)
    return containing_credential, secret_property_names, secret_names


def redact_credential(content, is_file=False):
    """Redact credential for the given content.

    :param content: The content(can be string or json object) or the file path.
    :type content: Any.
    :param is_file: flag to clarify the content is file path or not.
    :type is_file: bool.
    :returns: processed content.
    """
    if is_file:
        fp = content
        with open(fp, 'r') as f:
            content = f.read()
        content = redact_credential_for_string(content)
        with open(fp, 'w') as f:
            f.write(content)
        return fp

    if isinstance(content, str):
        return redact_credential_for_string(content)

    try:
        content = json.dumps(content)
        content = redact_credential_for_string(content)
        return json.loads(content)
    except ValueError:
        raise ValueError('The content is not string or json object.')


def redact_credential_for_string(content):
    return get_secret_masker().mask_secrets(content)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long
# pylint: disable=too-few-public-methods

import json
import unittest
import jmespath
from azure.cli.core.azclierror import (ResourceNotFoundError, ArgumentUsageError, InvalidArgumentValueError,
                                       MutuallyExclusiveArgumentError)
from azure.cli.testsdk import (ScenarioTest, ResourceGroupPreparer, JMESPathCheck, StorageAccountPreparer)
from knack.cli import CLIError
from knack.log import get_logger

logger = get_logger(__name__)

WINDOWS_ASP_LOCATION_WEBAPP = 'northeurope'
WINDOWS_ASP_LOCATION_FUNCTIONAPP = 'francecentral'
LINUX_ASP_LOCATION_WEBAPP = 'eastus2'
LINUX_ASP_LOCATION_FUNCTIONAPP = 'ukwest'


class FunctionAppAccessRestrictionScenarioTest(ScenarioTest):
    @ResourceGroupPreparer(parameter_name_for_location='location', location=WINDOWS_ASP_LOCATION_WEBAPP)
    @StorageAccountPreparer(location=WINDOWS_ASP_LOCATION_WEBAPP)
    def test_functionapp_access_restriction_show(self, resource_group, location):
        self.kwargs.update({
            'app_name': self.create_random_name(prefix='cli-funcapp-nwr', length=24),
            'loc': location
        })

        self.cmd('functionapp create -g {rg} -n {app_name} --consumption-plan-location {loc} -s {sa} --functions-version 4', checks=[
            JMESPathCheck('state', 'Running')
        ])

        self.cmd('functionapp config access-restriction show -g {rg} -n {app_name}', checks=[
            JMESPathCheck('length(@)', 5),
            JMESPathCheck('length(ipSecurityRestrictions)', 1),
            JMESPathCheck('ipSecurityRestrictions[0].name', 'Allow all'),
            JMESPathCheck('ipSecurityRestrictions[0].action', 'Allow'),
            JMESPathCheck('length(scmIpSecurityRestrictions)', 1),
            JMESPathCheck('scmIpSecurityRestrictions[0].name', 'Allow all'),
            JMESPathCheck('scmIpSecurityRestrictions[0].action', 'Allow'),
            JMESPathCheck('scmIpSecurityRestrictionsUseMain', False),
            JMESPathCheck('pSecurityRestrictionsDefaultAction', None),
            JMESPathCheck('scmIpSecurityRestrictionsDefaultAction', None)
        ])

    @ResourceGroupPreparer(parameter_name_for_location='location', location=WINDOWS_ASP_LOCATION_WEBAPP)
    @StorageAccountPreparer(location=WINDOWS_ASP_LOCATION_WEBAPP)
    def test_functionapp_access_restriction_set_simple(self, resource_group, location):
        self.kwargs.update({
            'app_name': self.create_random_name(prefix='cli-funcapp-nwr', length=24),
            'loc': location
        })

        self.cmd('functionapp create -g {rg} -n {app_name} --consumption-plan-location {loc} -s {sa} --functions-version 4', checks=[
            JMESPathCheck('state', 'Running')
        ])

        self.cmd('functionapp config access-restriction set -g {rg} -n {app_name} --use-same-restrictions-for-scm-site true', checks=[
            JMESPathCheck('scmIpSecurityRestrictionsUseMain', True)
        ])

        self.cmd('functionapp config access-restriction set -g {rg} -n {app_name} --default-action Deny', checks=[
            JMESPathCheck('ipSecurityRestrictionsDefaultAction', 'Deny'),
            JMESPathCheck('scmIpSecurityRestrictionsDefaultAction', None)
        ])

        self.cmd('functionapp config access-restriction set -g {rg} -n {app_name} --default-action Allow --scm-default-action Deny', checks=[
            JMESPathCheck('ipSecurityRestrictionsDefaultAction', 'Allow'),
            JMESPathCheck('scmIpSecurityRestrictionsDefaultAction', 'Deny')
        ])

        self.cmd('functionapp config access-restriction set -g {rg} -n {app_name} --default-action Deny', checks=[
            JMESPathCheck('ipSecurityRestrictionsDefaultAction', 'Deny'),
            JMESPathCheck('scmIpSecurityRestrictionsDefaultAction', 'Deny')
        ])

    @ResourceGroupPreparer(parameter_name_for_location='location', location=WINDOWS_ASP_LOCATION_WEBAPP)
    @StorageAccountPreparer(location=WINDOWS_ASP_LOCATION_WEBAPP)
    def test_functionapp_access_restriction_set_complex(self, resource_group, location):
        self.kwargs.update({
            'app_name': self.create_random_name(prefix='cli-funcapp-nwr', length=24),
            'loc': location
        })

        self.cmd('functionapp create -g {rg} -n {app_name} --consumption-plan-location {loc} -s {sa} --functions-version 4', checks=[
            JMESPathCheck('state', 'Running')
        ])

        self.cmd('functionapp config access-restriction set -g {rg} -n {app_name} --use-same-restrictions-for-scm-site', checks=[
            JMESPathCheck('scmIpSecurityRestrictionsUseMain', True)
        ])

        self.cmd('functionapp config access-restriction set -g {rg} -n {app_name} --use-same-restrictions-for-scm-site false', checks=[
            JMESPathCheck('scmIpSecurityRestrictionsUseMain', False)
        ])

    @ResourceGroupPreparer(random_name_length=17, parameter_name_for_location='location', location=WINDOWS_ASP_LOCATION_WEBAPP)
    # random_name_length is temporary until the bug fix in the API is deployed successfully & then should be removed.
    @StorageAccountPreparer(location=WINDOWS_ASP_LOCATION_WEBAPP)
    def test_functionapp_access_restriction_add(self, resource_group, location):
        self.kwargs.update({
            'app_name': self.create_random_name(prefix='cli-funcapp-nwr', length=24),
            'loc': location
        })

        self.cmd('functionapp create -g {rg} -n {app_name} --consumption-plan-location {loc} -s {sa} --functions-version 4', checks=[
            JMESPathCheck('state', 'Running')
        ])

        self.cmd('functionapp config access-restriction add -g {rg} -n {app_name} --rule-name developers --action Allow --ip-address 130.220.0.0/27 --priority 200', checks=[
            JMESPathCheck('length(@)', 2),
            JMESPathCheck('[0].name', 'developers'),
            JMESPathCheck('[0].action', 'Allow'),
            JMESPathCheck('[1].name', 'Deny all'),
            JMESPathCheck('[1].action', 'Deny')
        ])

    @ResourceGroupPreparer(parameter_name_for_location='location', location=WINDOWS_ASP_LOCATION_WEBAPP)
    @StorageAccountPreparer(location=WINDOWS_ASP_LOCATION_WEBAPP)
    def test_functionapp_access_restriction_add_ip_address_validation(self, resource_group, location):
        self.kwargs.update({
            'app_name': self.create_random_name(prefix='cli-funcapp-nwr', length=24),
            'loc': location
        })

        self.cmd('functionapp create -g {rg} -n {app_name} --consumption-plan-location {loc} -s {sa} --functions-version 4', checks=[
            JMESPathCheck('state', 'Running')
        ])

        self.cmd('functionapp config access-restriction add -g {rg} -n {app_name} --rule-name ipv4 --action Allow --ip-address 130.220.0.0 --priority 200', checks=[
            JMESPathCheck('length(@)', 2),
            JMESPathCheck('[0].name', 'ipv4'),
            JMESPathCheck('[0].action', 'Allow'),
            JMESPathCheck('[0].ipAddress', '130.220.0.0/32'),
            JMESPathCheck('[1].name', 'Deny all'),
            JMESPathCheck('[1].action', 'Deny')
        ])

        self.cmd('functionapp config access-restriction add -g {rg} -n {app_name} --rule-name ipv6 --action Allow --ip-address 2004::1000 --priority 200', checks=[
            JMESPathCheck('length(@)', 3),
            JMESPathCheck('[1].name', 'ipv6'),
            JMESPathCheck('[1].action', 'Allow'),
            JMESPathCheck('[1].ipAddress', '2004::1000/128')
        ])

    @unittest.skip("Invalid test case that cannot pass in the live mode.")
    @ResourceGroupPreparer(parameter_name_for_location='location', location=WINDOWS_ASP_LOCATION_WEBAPP)
    @StorageAccountPreparer(location=WINDOWS_ASP_LOCATION_WEBAPP)
    def test_functionapp_access_restriction_add_service_endpoint(self, resource_group, location):
        self.kwargs.update({
            'app_name': self.create_random_name(prefix='cli-funcapp-nwr', length=24),
            'plan_name': self.create_random_name(prefix='cli-plan-nwr', length=24),
            'vnet_name': self.create_random_name(prefix='cli-vnet-nwr', length=24)
        })

        self.cmd('appservice plan create -g {rg} -n {plan_name}')
        self.cmd('functionapp create -g {rg} -n {app_name} --plan {plan_name} -s {sa}', checks=[
            JMESPathCheck('state', 'Running')
        ])

        self.cmd('az network vnet create -g {rg} -n {vnet_name} --address-prefixes 10.0.0.0/16 --subnet-name endpoint-subnet --subnet-prefixes 10.0.0.0/24', checks=[
            JMESPathCheck('subnets[0].serviceEndpoints', None)
        ])

        self.cmd('functionapp config access-restriction add -g {rg} -n {app_name} --rule-name vnet-integration --action Allow --vnet-name {vnet_name} --subnet endpoint-subnet --priority 150', checks=[
            JMESPathCheck('length(@)', 2),
            JMESPathCheck('[0].name', 'vnet-integration'),
            JMESPathCheck('[0].action', 'Allow'),
            JMESPathCheck('[1].name', 'Deny all'),
            JMESPathCheck('[1].action', 'Deny')
        ])

    @ResourceGroupPreparer(parameter_name_for_location='location', location=WINDOWS_ASP_LOCATION_WEBAPP)
    @StorageAccountPreparer(location=WINDOWS_ASP_LOCATION_WEBAPP)
    def test_functionapp_access_restriction_remove(self, resource_group, location):
        self.kwargs.update({
            'app_name': self.create_random_name(prefix='cli-funcapp-nwr', length=24),
            'loc': location
        })

        self.cmd('functionapp create -g {rg} -n {app_name} --consumption-plan-location {loc} -s {sa} --functions-version 4', checks=[
            JMESPathCheck('state', 'Running')
        ])

        self.cmd('functionapp config access-restriction add -g {rg} -n {app_name} --rule-name developers --action Allow --ip-address 130.220.0.0/27 --priority 200', checks=[
            JMESPathCheck('length(@)', 2),
            JMESPathCheck('[0].name', 'developers'),
            JMESPathCheck('[0].action', 'Allow'),
            JMESPathCheck('[1].name', 'Deny all'),
            JMESPathCheck('[1].action', 'Deny')
        ])

        self.cmd('functionapp config access-restriction remove -g {rg} -n {app_name} --rule-name developers', checks=[
            JMESPathCheck('length(@)', 1),
            JMESPathCheck('[0].name', 'Allow all'),
            JMESPathCheck('[0].action', 'Allow')
        ])

    @ResourceGroupPreparer(parameter_name_for_location='location', location=WINDOWS_ASP_LOCATION_WEBAPP)
    @StorageAccountPreparer(location=WINDOWS_ASP_LOCATION_WEBAPP)
    def test_functionapp_access_restriction_add_scm(self, resource_group, location):
        self.kwargs.update({
            'app_name': self.create_random_name(prefix='cli-funcapp-nwr', length=24),
            'loc': location
        })

        self.cmd('functionapp create -g {rg} -n {app_name} --consumption-plan-location {loc} -s {sa} --functions-version 4', checks=[
            JMESPathCheck('state', 'Running')
        ])

        self.cmd('functionapp config access-restriction add -g {rg} -n {app_name} --rule-name developers --action Allow --ip-address 130.220.0.0/27 --priority 200 --scm-site', checks=[
            JMESPathCheck('length(@)', 2),
            JMESPathCheck('[0].name', 'developers'),
            JMESPathCheck('[0].action', 'Allow'),
            JMESPathCheck('[1].name', 'Deny all'),
            JMESPathCheck('[1].action', 'Deny')
        ])

    @ResourceGroupPreparer(parameter_name_for_location='location', location=WINDOWS_ASP_LOCATION_WEBAPP)
    @StorageAccountPreparer(location=WINDOWS_ASP_LOCATION_WEBAPP)
    def test_functionapp_access_restriction_remove_scm(self, resource_group, location):
        self.kwargs.update({
            'app_name': self.create_random_name(prefix='cli-funcapp-nwr', length=24),
            'loc': location
        })

        self.cmd('functionapp create -g {rg} -n {app_name} --consumption-plan-location {loc} -s {sa} --functions-version 4', checks=[
            JMESPathCheck('state', 'Running')
        ])

        self.cmd('functionapp config access-restriction add -g {rg} -n {app_name} --rule-name developers --action Allow --ip-address 130.220.0.0/27 --priority 200 --scm-site', checks=[
            JMESPathCheck('length(@)', 2),
            JMESPathCheck('[0].name', 'developers'),
            JMESPathCheck('[0].action', 'Allow'),
            JMESPathCheck('[1].name', 'Deny all'),
            JMESPathCheck('[1].action', 'Deny')
        ])

        self.cmd('functionapp config access-restriction remove -g {rg} -n {app_name} --rule-name developers --scm-site', checks=[
            JMESPathCheck('length(@)', 1),
            JMESPathCheck('[0].name', 'Allow all'),
            JMESPathCheck('[0].action', 'Allow')
        ])

    @unittest.skip("Function app slot shouldn't use webapp")
    @ResourceGroupPreparer(parameter_name_for_location='location', location=WINDOWS_ASP_LOCATION_WEBAPP)
    @StorageAccountPreparer(location=WINDOWS_ASP_LOCATION_WEBAPP)
    def test_functionapp_access_restriction_slot(self, resource_group, location):
        self.kwargs.update({
            'app_name': self.create_random_name(prefix='cli-funcapp-nwr', length=24),
            'loc': location,
            'slot_name': 'stage'
        })

        self.cmd('functionapp create -g {rg} -n {app_name} --consumption-plan-location {loc} -s {sa}', checks=[
            JMESPathCheck('state', 'Running')
        ])
        self.cmd('functionapp deployment slot create -g {rg} -n {app_name} --slot {slot_name}', checks=[
            JMESPathCheck('state', 'Running')
        ])

        self.cmd('functionapp config access-restriction show -g {rg} -n {app_name} --slot {slot_name}', checks=[
            JMESPathCheck('length(@)', 5),
            JMESPathCheck('length(ipSecurityRestrictions)', 1),
            JMESPathCheck('ipSecurityRestrictions[0].name', 'Allow all'),
            JMESPathCheck('ipSecurityRestrictions[0].action', 'Allow'),
            JMESPathCheck('length(scmIpSecurityRestrictions)', 1),
            JMESPathCheck('scmIpSecurityRestrictions[0].name', 'Allow all'),
            JMESPathCheck('scmIpSecurityRestrictions[0].action', 'Allow'),
            JMESPathCheck('scmIpSecurityRestrictionsUseMain', False),
            JMESPathCheck('pSecurityRestrictionsDefaultAction', None),
            JMESPathCheck('scmIpSecurityRestrictionsDefaultAction', None)
        ])

        self.cmd('functionapp config access-restriction add -g {rg} -n {app_name} --rule-name developers --action Allow --ip-address 130.220.0.0/27 --priority 200 --slot {slot_name}', checks=[
            JMESPathCheck('length(@)', 2),
            JMESPathCheck('[0].name', 'developers'),
            JMESPathCheck('[0].action', 'Allow'),
            JMESPathCheck('[1].name', 'Deny all'),
            JMESPathCheck('[1].action', 'Deny')
        ])


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import re
import unittest
from unittest import mock

from azure.cli.core.auth.identity import (Identity, ServicePrincipalAuth, ServicePrincipalStore,
                                          _get_authority_url)
from knack.util import CLIError

# CERTIFICATE section in sp_cert.pem
PUBLIC_CERTIFICATE = """-----BEGIN CERTIFICATE-----
MIIDtTCCAp2gAwIBAgIJAPMNsT0qjg1ZMA0GCSqGSIb3DQEBBQUAMEUxCzAJBgNV
BAYTAkFVMRMwEQYDVQQIEwpTb21lLVN0YXRlMSEwHwYDVQQKExhJbnRlcm5ldCBX
aWRnaXRzIFB0eSBMdGQwHhcNMTcwMzEwMDQ0NjEyWhcNMTgwMzEwMDQ0NjEyWjBF
MQswCQYDVQQGEwJBVTETMBEGA1UECBMKU29tZS1TdGF0ZTEhMB8GA1UEChMYSW50
ZXJuZXQgV2lkZ2l0cyBQdHkgTHRkMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIB
CgKCAQEAxec32tnXNiPz2WBTpv7ccZvYqBR2Gr8vimQbiNgT3aHY/dzV26pYv/88
X5PbkibAr3YXJP64nGI/0MGvFWYi6c6C0Ar6QL/MgRLIGIO8JePTxKu9ZDx+5Crw
beJRQgz7nEtCWsIx5WiIx5/yjUR5AqrNwSxNWo6Ct3E1YWzGyI03gEEr82tEG9Vd
ObIRq05v1hHKTm27xln41JZI1aUMzd/K/pckb6nQLtV6OpOmzZQILMOV95SKJ8+k
1gnxfOX2t9JPgTuiVmwvgYLb1k7Hfqs1/KZt4IyIRkBaXPy2j5Guz09uR1Dg4tOc
oSPwDeN0aQQSucRsk0iaof3DXMfVLQIDAQABo4GnMIGkMB0GA1UdDgQWBBRpCyBM
VgNXHqX5MrBdAQ1Hzf8l7jB1BgNVHSMEbjBsgBRpCyBMVgNXHqX5MrBdAQ1Hzf8l
7qFJpEcwRTELMAkGA1UEBhMCQVUxEzARBgNVBAgTClNvbWUtU3RhdGUxITAfBgNV
BAoTGEludGVybmV0IFdpZGdpdHMgUHR5IEx0ZIIJAPMNsT0qjg1ZMAwGA1UdEwQF
MAMBAf8wDQYJKoZIhvcNAQEFBQADggEBAEH/nmErQLSxsMDk3LgTpBY6ibl6xU0k
Lt1wbC+Z3sgpt82oA4BiulcJtTf3IrvBXJNRaB++ChjqRnK8O6uWbBQxvz/V8l+9
g3s49VSaX3QB74Rh1NIfKhUyYlG3yi8qBJA6tlCNNXGQoYvND9Y3gorj+LzH3Eqf
9g2oBm2jWaiPBHjuuUbd+SBS2hQn/i2huWnz1yewrtfVpRwWrQQHa1Qv3ivKDK2H
2LOdn2Xs3/ZGsi1ySfjzxjTbuPhUaEUy+ZfV2dgmqiS//BAWI5opo7TgeplrGk2P
h5Fwbt0FxaqFCNZdrPI7FRnbKZwvGx0A+Zj8ZpNjft3QjuUg+xqMKMs=
-----END CERTIFICATE-----"""


TEST_CERT = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'sp_cert.pem')

with open(TEST_CERT) as f:
    CERTIFICATE_STRING = f.read()


class TestIdentity(unittest.TestCase):

    @mock.patch("azure.cli.core.auth.identity.ServicePrincipalStore.save_entry")
    @mock.patch("msal.application.ConfidentialClientApplication.acquire_token_for_client")
    @mock.patch("msal.application.ConfidentialClientApplication.__init__", return_value=None)
    def test_login_with_service_principal_secret(self, init_mock, acquire_token_for_client_mock,
                                                 save_entry_mock):
        acquire_token_for_client_mock.return_value = {'access_token': "test_token"}

        identity = Identity('https://login.microsoftonline.com', tenant_id='tenant1')
        identity.login_with_service_principal("sp_id1", {"client_secret": "test_secret"}, "openid")

        assert init_mock.call_args.args == ('sp_id1',)
        assert init_mock.call_args.kwargs['client_credential'] == 'test_secret'
        assert init_mock.call_args.kwargs['authority'] == 'https://login.microsoftonline.com/tenant1'

        assert save_entry_mock.call_args.args[0] == {
            'client_id': 'sp_id1',
            'tenant': 'tenant1',
            'client_secret': 'test_secret'
        }

    @mock.patch("azure.cli.core.auth.identity.ServicePrincipalStore.save_entry")
    @mock.patch("msal.application.ConfidentialClientApplication.acquire_token_for_client")
    @mock.patch("msal.application.ConfidentialClientApplication.__init__", return_value=None)
    def test_login_with_service_principal_certificate(self, init_mock, acquire_token_for_client_mock,
                                                      save_entry_mock):
        acquire_token_for_client_mock.return_value = {'access_token': "test_token"}

        identity = Identity('https://login.microsoftonline.com', tenant_id='tenant1')
        identity.login_with_service_principal("sp_id1", {'certificate': TEST_CERT}, 'openid')

        assert init_mock.call_args.args == ('sp_id1',)
        assert init_mock.call_args.kwargs['client_credential'] == {
                'private_key': CERTIFICATE_STRING,
                'thumbprint': 'F06A53848BBE714A4290D69D335279C1D01073FD'
            }
        assert init_mock.call_args.kwargs['authority'] == 'https://login.microsoftonline.com/tenant1'

        assert save_entry_mock.call_args[0][0] == {
            'client_id': 'sp_id1',
            'tenant': 'tenant1',
            'certificate': TEST_CERT
        }

    @mock.patch("azure.cli.core.auth.identity.ServicePrincipalStore.save_entry")
    @mock.patch("msal.application.ConfidentialClientApplication.acquire_token_for_client")
    @mock.patch("msal.application.ConfidentialClientApplication.__init__", return_value=None)
    def test_login_with_service_principal_certificate_sn_issuer(self, init_mock, acquire_token_for_client_mock,
                                                                save_entry_mock):
        acquire_token_for_client_mock.return_value = {'access_token': "test_token"}

        identity = Identity('https://login.microsoftonline.com', tenant_id='tenant1')
        identity.login_with_service_principal("sp_id1",
                                              {
                                                  'certificate': TEST_CERT,
                                                  'use_cert_sn_issuer': True,
                                              }, "openid")

        assert init_mock.call_args.args == ('sp_id1',)
        assert init_mock.call_args.kwargs['client_credential'] == {
            "private_key": CERTIFICATE_STRING,
            "thumbprint": 'F06A53848BBE714A4290D69D335279C1D01073FD',
            "public_certificate": PUBLIC_CERTIFICATE
        }
        assert init_mock.call_args.kwargs['authority'] == 'https://login.microsoftonline.com/tenant1'

        assert save_entry_mock.call_args.args[0] == {
            'client_id': 'sp_id1',
            'tenant': 'tenant1',
            'certificate': TEST_CERT,
            'use_cert_sn_issuer': True
        }

    def test_login_with_service_principal_certificate_cert_err(self):
        import os
        identity = Identity('https://login.microsoftonline.com')
        current_dir = os.path.dirname(os.path.realpath(__file__))
        test_cert_file = os.path.join(current_dir, 'err_sp_cert.pem')

        with self.assertRaisesRegex(CLIError, "Invalid certificate"):
            identity.login_with_service_principal("sp_id1", {"certificate": test_cert_file}, "openid")

    @mock.patch("azure.cli.core.auth.identity.ServicePrincipalStore.save_entry")
    @mock.patch("msal.application.ConfidentialClientApplication.acquire_token_for_client")
    @mock.patch("msal.application.ConfidentialClientApplication.__init__", return_value=None)
    def test_login_with_service_principal_client_assertion(self, init_mock, acquire_token_for_client_mock,
                                                           save_entry_mock):
        acquire_token_for_client_mock.return_value = {'access_token': "test_token"}

        identity = Identity('https://login.microsoftonline.com', tenant_id='tenant1')
        identity.login_with_service_principal("sp_id1", {'client_assertion': 'test_jwt'}, "openid")

        assert init_mock.call_args.args == ('sp_id1',)
        assert init_mock.call_args.kwargs['client_credential'] == {"client_assertion": 'test_jwt'}
        assert init_mock.call_args.kwargs['authority'] == 'https://login.microsoftonline.com/tenant1'

        assert save_entry_mock.call_args.args[0] == {
            'client_id': 'sp_id1',
            'tenant': 'tenant1',
            'client_assertion': 'test_jwt',
        }

    @mock.patch("msal.application.PublicClientApplication.remove_account")
    @mock.patch("msal.application.PublicClientApplication.get_accounts")
    def test_logout_user(self, get_accounts_mock, remove_account_mock):
        accounts = [
            {
                'home_account_id': '00000000-0000-0000-0000-000000000000.00000000-0000-0000-0000-000000000000',
                'environment': 'login.microsoftonline.com',
                'username': 'test@test.com',
                'account_source': 'broker',
                'authority_type': 'MSSTS',
                'local_account_id': '00000000-0000-0000-0000-000000000000',
                'realm': '00000000-0000-0000-0000-000000000000'
            }
        ]
        get_accounts_mock.return_value = accounts

        identity = Identity('https://login.microsoftonline.com')
        identity.logout_user('test@test.com')
        remove_account_mock.assert_called_with(accounts[0])

    @mock.patch("azure.cli.core.auth.identity.ServicePrincipalStore.remove_entry")
    @mock.patch("msal.application.ConfidentialClientApplication.remove_tokens_for_client")
    @mock.patch("msal.application.ConfidentialClientApplication.__init__", return_value=None)
    def test_logout_service_principal(self, init_mock, remove_tokens_for_client_mock, remove_entry_mock):
        identity = Identity('https://login.microsoftonline.com')
        client_id = 'sp_id1'
        identity.logout_service_principal(client_id)
        assert init_mock.call_args.args[0] == client_id
        remove_tokens_for_client_mock.assert_called_once()
        remove_entry_mock.assert_called_with(client_id)


class TestServicePrincipalAuth(unittest.TestCase):

    def test_service_principal_auth_client_secret(self):
        sp_auth = ServicePrincipalAuth.build_from_credential('tenant1', 'sp_id1', {'client_secret': "test_secret"})

        # Verify persist entry
        entry = sp_auth.get_entry_to_persist()
        assert entry == {
            'client_id': 'sp_id1',
            'tenant': 'tenant1',
            'client_secret': 'test_secret'
        }

        # Verify msal client_credential
        client_credential = sp_auth.get_msal_client_credential()
        assert client_credential == 'test_secret'

    def test_service_principal_auth_certificate(self):
        sp_auth = ServicePrincipalAuth.build_from_credential('tenant1', 'sp_id1', {'certificate': TEST_CERT})

        # To compute the thumbprint:
        #   openssl x509 -in sp_cert.pem -noout -fingerprint
        assert sp_auth._thumbprint == 'F06A53848BBE714A4290D69D335279C1D01073FD'

        # Verify persist entry
        entry = sp_auth.get_entry_to_persist()
        assert entry == {
            'client_id': 'sp_id1',
            'tenant': 'tenant1',
            'certificate': TEST_CERT
        }

        # Verify msal client_credential
        client_credential = sp_auth.get_msal_client_credential()
        assert client_credential == {
            'private_key': CERTIFICATE_STRING,
            'thumbprint': 'F06A53848BBE714A4290D69D335279C1D01073FD'
        }

    def test_service_principal_auth_certificate_sn_issuer(self):
        sp_auth = ServicePrincipalAuth.build_from_credential('tenant1', 'sp_id1',
                                                             {
                                                                 'certificate': TEST_CERT,
                                                                 'use_cert_sn_issuer': True,
                                                             })

        # To compute the thumbprint:
        #   openssl x509 -in sp_cert.pem -noout -fingerprint
        assert sp_auth._thumbprint == 'F06A53848BBE714A4290D69D335279C1D01073FD'
        assert sp_auth._public_certificate == PUBLIC_CERTIFICATE

        # Verify persist entry
        entry = sp_auth.get_entry_to_persist()
        assert entry == {
            'client_id': 'sp_id1',
            'tenant': 'tenant1',
            'certificate': TEST_CERT,
            'use_cert_sn_issuer': True,
        }

        # Verify msal client_credential
        client_credential = sp_auth.get_msal_client_credential()
        assert client_credential == {
            'private_key': CERTIFICATE_STRING,
            'thumbprint': 'F06A53848BBE714A4290D69D335279C1D01073FD',
            'public_certificate': PUBLIC_CERTIFICATE
        }

    def test_service_principal_auth_client_assertion(self):
        sp_auth = ServicePrincipalAuth.build_from_credential('tenant1', 'sp_id1',
                                                             {'client_assertion': 'test_jwt'})
        assert sp_auth.client_assertion == 'test_jwt'

        # Verify persist entry
        entry = sp_auth.get_entry_to_persist()
        assert entry == {
            'client_id': 'sp_id1',
            'tenant': 'tenant1',
            'client_assertion': 'test_jwt'
        }

        # Verify msal client_credential
        client_credential = sp_auth.get_msal_client_credential()
        assert client_credential == {'client_assertion': 'test_jwt'}

    def test_build_credential(self):
        # client_secret
        cred = ServicePrincipalAuth.build_credential(client_secret="test_secret")
        assert cred == {"client_secret": "test_secret"}

        # certificate
        current_dir = os.path.dirname(os.path.realpath(__file__))
        test_cert_file = os.path.join(current_dir, 'sp_cert.pem')
        cred = ServicePrincipalAuth.build_credential(certificate=test_cert_file)
        assert cred == {'certificate': test_cert_file}

        # certificate path with '~', which expands to HOME folder
        import shutil
        home = os.path.expanduser('~')
        home_cert = os.path.join(home, 'sp_cert.pem')  # C:\Users\username\sp_cert.pem
        shutil.copyfile(test_cert_file, home_cert)
        cred = ServicePrincipalAuth.build_credential(certificate=os.path.join('~', 'sp_cert.pem'))  # ~\sp_cert.pem
        assert cred == {'certificate': home_cert}
        os.remove(home_cert)

        # Certificate with use_cert_sn_issuer=True
        cred = ServicePrincipalAuth.build_credential(certificate=test_cert_file, use_cert_sn_issuer=True)
        assert cred == {'certificate': test_cert_file, 'use_cert_sn_issuer': True}

        # client_assertion
        cred = ServicePrincipalAuth.build_credential(client_assertion="test_jwt")
        assert cred == {"client_assertion": "test_jwt"}


class TestServicePrincipalStore(unittest.TestCase):

    test_sp = {
        'client_id': 'myapp',
        'tenant': 'mytenant',
        'client_secret': 'test_secret'
    }

    def test_load_entry(self):
        store = MemoryStore()

        secret_store = ServicePrincipalStore(store)
        store._content = [self.test_sp]

        entry = secret_store.load_entry("myapp", "mytenant")
        self.assertEqual(entry['client_secret'], "test_secret")

    def test_save_entry(self):
        store = MemoryStore()

        secret_store = ServicePrincipalStore(store)
        secret_store.save_entry(self.test_sp)

        assert store._content == [self.test_sp]

    def test_save_entry_add_new(self):
        store = MemoryStore()

        test_sp2 = {
            'client_id': "myapp2",
            'tenant': "mytenant2",
            'client_secret': "test_secret2"
        }

        store._content = [self.test_sp]
        secret_store = ServicePrincipalStore(store)
        secret_store.save_entry(test_sp2)
        assert store._content == [self.test_sp, test_sp2]

    def test_save_entry_update_existing(self):
        store = MemoryStore()

        store._content = [self.test_sp]
        new_creds = self.test_sp.copy()
        new_creds['client_secret'] = 'test_secret'

        secret_store = ServicePrincipalStore(store)
        secret_store.save_entry(new_creds)
        assert store._content == [new_creds]

    def test_remove_entry(self):
        store = MemoryStore()

        store._content = [self.test_sp]
        secret_store = ServicePrincipalStore(store)
        secret_store.remove_entry('myapp')
        assert store._content == []


class TestUtils(unittest.TestCase):

    def test_get_authority_url(self):
        # AAD
        # Default tenant
        self.assertEqual(_get_authority_url('https://login.microsoftonline.com', None),
                         ('https://login.microsoftonline.com/organizations', False))
        # Trailing slash is stripped
        self.assertEqual(_get_authority_url('https://login.microsoftonline.com/', None),
                         ('https://login.microsoftonline.com/organizations', False))
        # Custom tenant
        self.assertEqual(_get_authority_url('https://login.microsoftonline.com',
                                            '54826b22-38d6-4fb2-bad9-b7b93a3e9c5a'),
                         ('https://login.microsoftonline.com/54826b22-38d6-4fb2-bad9-b7b93a3e9c5a', False))

        # ADFS
        # Default tenant
        adfs_expected = ('https://adfs.redmond.azurestack.corp.microsoft.com/adfs', True)
        self.assertEqual(_get_authority_url('https://adfs.redmond.azurestack.corp.microsoft.com/adfs', None),
                         adfs_expected)
        # Trailing slash is stripped
        self.assertEqual(_get_authority_url('https://adfs.redmond.azurestack.corp.microsoft.com/adfs/', None),
                         adfs_expected)
        # Tenant ID is discarded
        self.assertEqual(_get_authority_url('https://adfs.redmond.azurestack.corp.microsoft.com/adfs',
                                            '601d729d-0000-0000-0000-000000000000'),
                         adfs_expected)


class MemoryStore:

    def __init__(self):
        self._content = []

    def save(self, content):
        self._content = content

    def load(self):
        return self._content


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from azure.cli.command_modules.acs._client_factory import get_container_service_client
from azure.cli.command_modules.acs.azuremonitormetrics.addonput import addon_put
from azure.cli.command_modules.acs.azuremonitormetrics.amg.link import link_grafana_instance
from azure.cli.command_modules.acs.azuremonitormetrics.amw.helper import get_azure_monitor_workspace_resource
from azure.cli.command_modules.acs.azuremonitormetrics.dc.dce_api import create_dce
from azure.cli.command_modules.acs.azuremonitormetrics.dc.dcr_api import create_dcr
from azure.cli.command_modules.acs.azuremonitormetrics.dc.dcra_api import create_dcra
from azure.cli.command_modules.acs.azuremonitormetrics.dc.delete import (
    delete_dc_objects_if_prometheus_enabled,
    get_dc_objects_list
)
from azure.cli.command_modules.acs.azuremonitormetrics.helper import (
    check_azuremonitormetrics_profile,
    rp_registrations
)
from azure.cli.command_modules.acs.azuremonitormetrics.recordingrules.create import create_rules
from azure.cli.command_modules.acs.azuremonitormetrics.recordingrules.delete import delete_rules
from azure.cli.core.azclierror import CLIError, InvalidArgumentValueError, UnknownError
from knack.log import get_logger


logger = get_logger(__name__)


# pylint: disable=line-too-long
def _addon_put_with_control_plane(cmd, cluster_subscription, cluster_resource_group_name, cluster_name):
    """Sibling of ``addon_put`` that ALSO flips ``metrics.controlPlane.enabled=True``.

    Used by the greenfield ``aks create --enable-control-plane-metrics`` path. The
    initial cluster PUT intentionally leaves ``control_plane`` unset so the RP does
    not schedule the control-plane-metrics collection (CCP) pod before the DCRA is
    created in postprocessing. Once the DCRA exists, we issue this PUT so the CCP
    pod is scheduled with its DCRA already in place (race-free).
    """
    client = get_container_service_client(cmd.cli_ctx, cluster_subscription).managed_clusters
    try:
        mc = client.get(cluster_resource_group_name, cluster_name)
    except CLIError as e:
        raise UnknownError(str(e)) from e
    # Enable metrics if present and not already enabled (mirrors addon_put).
    if hasattr(mc, "azure_monitor_profile") and mc.azure_monitor_profile:
        if hasattr(mc.azure_monitor_profile, "metrics") and mc.azure_monitor_profile.metrics:
            if getattr(mc.azure_monitor_profile.metrics, "enabled", None) is False:
                mc.azure_monitor_profile.metrics.enabled = True
            # Flip control plane now that DCRA exists.
            try:
                from azure.mgmt.containerservice.models import (
                    ManagedClusterAzureMonitorProfileMetricsControlPlane,
                )
                mc.azure_monitor_profile.metrics.control_plane = (
                    ManagedClusterAzureMonitorProfileMetricsControlPlane(enabled=True)
                )
            except ImportError:
                # Fallback for SDK versions that don't expose the model directly:
                # set a dict that the generated client will serialize as the property.
                mc.azure_monitor_profile.metrics.control_plane = {"enabled": True}
    # Unlike the sibling ``addon_put`` (where ``metrics.enabled`` is already true on the
    # cluster from the initial PUT and the postprocessing PUT is just a safety re-affirm),
    # this is the ONLY place where ``controlPlane.enabled`` is set during ``aks create``.
    # Wait for the LRO so the CP flip is durably persisted before the create command
    # returns; otherwise callers / tests that read the cluster immediately could see the
    # pre-flip state.
    try:
        poller = client.begin_create_or_update(cluster_resource_group_name, cluster_name, mc)
        poller.result()
    except Exception as e:
        raise UnknownError(str(e)) from e


# pylint: disable=line-too-long
def link_azure_monitor_profile_artifacts(
        cmd,
        cluster_subscription,
        cluster_resource_group_name,
        cluster_name,
        cluster_region,
        raw_parameters,
        create_flow
):
    # MAC creation if required
    azure_monitor_workspace_resource_id, azure_monitor_workspace_location = get_azure_monitor_workspace_resource(cmd, cluster_subscription, cluster_region, raw_parameters)
    # DCE creation
    dce_resource_id = create_dce(cmd, cluster_subscription, cluster_resource_group_name, cluster_name, azure_monitor_workspace_location)
    # DCR creation
    dcr_resource_id = create_dcr(cmd, azure_monitor_workspace_location, azure_monitor_workspace_resource_id, cluster_subscription, cluster_resource_group_name, cluster_name, dce_resource_id)
    # DCRA creation
    create_dcra(cmd, cluster_region, cluster_subscription, cluster_resource_group_name, cluster_name, dcr_resource_id)
    # Link grafana
    link_grafana_instance(cmd, raw_parameters, azure_monitor_workspace_resource_id)
    # create recording rules and alerts
    create_rules(cmd, cluster_subscription, cluster_resource_group_name, cluster_name, azure_monitor_workspace_resource_id, azure_monitor_workspace_location, raw_parameters)
    # if aks cluster create flow -> do a PUT on the AKS cluster to enable the addon
    if create_flow:
        # If --enable-control-plane-metrics was specified on create, flip
        # metrics.controlPlane.enabled HERE (after DCRA creation) instead of on
        # the initial cluster PUT. This avoids the CCP pod being scheduled before
        # its DCRA exists (which would cause CrashLoopBackOff until reconciliation).
        enable_cp = bool(raw_parameters and raw_parameters.get("enable_control_plane_metrics"))
        if enable_cp:
            _addon_put_with_control_plane(cmd, cluster_subscription, cluster_resource_group_name, cluster_name)
        else:
            addon_put(cmd, cluster_subscription, cluster_resource_group_name, cluster_name)


# pylint: disable=line-too-long
def unlink_azure_monitor_profile_artifacts(cmd, cluster_subscription, cluster_resource_group_name, cluster_name):
    logger.warning("Deleting all custom resources for the `azmonitoring.coreos.com` custom resource defintion created by the Managed Prometheus addon")
    # Remove DC* if prometheus is enabled
    dc_objects_list = get_dc_objects_list(cmd, cluster_subscription, cluster_resource_group_name, cluster_name)
    delete_dc_objects_if_prometheus_enabled(cmd, dc_objects_list, cluster_subscription, cluster_resource_group_name, cluster_name)
    # Delete rules (Conflict({"error":{"code":"InvalidResourceLocation","message":"The resource 'NodeRecordingRulesRuleGroup-<clustername>' already exists in location 'eastus2' in resource group '<clustername>'.
    # A resource with the same name cannot be created in location 'eastus'. Please select a new resource name."}})
    delete_rules(cmd, cluster_subscription, cluster_resource_group_name, cluster_name)


# pylint: disable=too-many-locals,too-many-branches,too-many-statements,line-too-long
def ensure_azure_monitor_profile_prerequisites(
    cmd,
    cluster_subscription,
    cluster_resource_group_name,
    cluster_name,
    cluster_region,
    raw_parameters,
    remove_azuremonitormetrics,
    create_flow=False
):
    cloud_name = cmd.cli_ctx.cloud.name
    if cloud_name.lower() == "ussec" or cloud_name.lower() == "usdod":
        grafana_resource_id = raw_parameters.get("grafana_resource_id")
        if grafana_resource_id is not None:
            if grafana_resource_id != "":
                raise InvalidArgumentValueError(f"{cloud_name} does not support Azure Managed Grafana yet.")

    if remove_azuremonitormetrics:
        unlink_azure_monitor_profile_artifacts(
            cmd,
            cluster_subscription,
            cluster_resource_group_name,
            cluster_name
        )
    else:
        is_prometheus_enabled = False
        # Check if already onboarded
        if create_flow is False:
            is_prometheus_enabled = check_azuremonitormetrics_profile(cmd, cluster_subscription, cluster_resource_group_name, cluster_name)
        if is_prometheus_enabled:
            print("Azure Prometheus is already enabled : This command will only allow updates to the KSM parameters. All other parameters will be ignored")
        # Do RP registrations and artifact creation (DC*, rules, grafana link etc.) if not enabled already
        # Otherwise move forward so that the addon can be enabled with new KSM parameters
        if is_prometheus_enabled is False:
            rp_registrations(cmd, cluster_subscription, raw_parameters)
            link_azure_monitor_profile_artifacts(
                cmd,
                cluster_subscription,
                cluster_resource_group_name,
                cluster_name,
                cluster_region,
                raw_parameters,
                create_flow
            )



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=unused-argument, line-too-long
from azure.cli.core.util import CLIError, sdk_no_wait
from .._client_factory import cf_postgres_flexible_servers
from ..utils._flexible_server_util import get_tenant_id
from ..utils.validators import validate_resource_group


def flexible_server_microsoft_entra_admin_set(cmd, client, resource_group_name, server_name, login, sid, principal_type=None, no_wait=False):
    validate_resource_group(resource_group_name)

    server_operations_client = cf_postgres_flexible_servers(cmd.cli_ctx, '_')

    instance = server_operations_client.get(resource_group_name, server_name)

    if 'replica' in instance.replication_role.lower():
        raise CLIError("Cannot create a Microsoft Entra admin on a server with replication role. Use the primary server instead.")

    return _create_admin(client, resource_group_name, server_name, login, sid, principal_type, no_wait)


def _create_admin(client, resource_group_name, server_name, principal_name, sid, principal_type=None, no_wait=False):
    parameters = {
        'properties': {
            'principalName': principal_name,
            'tenantId': get_tenant_id(),
            'principalType': principal_type
        }
    }

    return sdk_no_wait(no_wait, client.begin_create_or_update, resource_group_name, server_name, sid, parameters)


def flexible_server_microsoft_entra_admin_delete(cmd, client, resource_group_name, server_name, sid, no_wait=False):
    validate_resource_group(resource_group_name)

    server_operations_client = cf_postgres_flexible_servers(cmd.cli_ctx, '_')

    instance = server_operations_client.get(resource_group_name, server_name)

    if 'replica' in instance.replication_role.lower():
        raise CLIError("Cannot delete an Microsoft Entra admin on a server with replication role. Use the primary server instead.")

    return sdk_no_wait(no_wait, client.begin_delete, resource_group_name, server_name, sid)


def flexible_server_microsoft_entra_admin_list(client, resource_group_name, server_name):
    validate_resource_group(resource_group_name)

    return client.list_by_server(
        resource_group_name=resource_group_name,
        server_name=server_name)


def flexible_server_microsoft_entra_admin_show(client, resource_group_name, server_name, sid):
    validate_resource_group(resource_group_name)

    return client.get(
        resource_group_name=resource_group_name,
        server_name=server_name,
        object_id=sid)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from knack.log import get_logger
from azure.core import PipelineClient
from azure.core.configuration import Configuration
from azure.core.polling.base_polling import LocationPolling, StatusCheckPolling
from abc import abstractmethod

from azure.cli.core.cloud import (CloudEndpointNotSetException, CloudSuffixNotSetException,
                                  CloudNameEnum as _CloudNameEnum)


logger = get_logger(__name__)

registered_clients = {}
CloudNameEnum = _CloudNameEnum  # redefine CloudNameEnum in aaz to decouple with cloud for aaz generated commands.

# pylint: disable=too-many-instance-attributes, too-few-public-methods, protected-access


def register_client(name):
    def decorator(cls):
        if name in registered_clients:
            assert registered_clients[name] == cls
        else:
            registered_clients[name] = cls
        return cls

    return decorator


class AAZClientConfiguration(Configuration):
    def __init__(self, credential, credential_scopes, **kwargs):
        if credential is None:
            raise ValueError("Parameter 'credential' must not be None.")
        if not credential_scopes:
            raise ValueError("Parameter 'credential_scopes' is required.")
        super().__init__(**kwargs)
        self.credential = credential
        self.credential_scopes = credential_scopes
        self._configure(**kwargs)

    def _configure(
            self,
            **kwargs
    ):
        from azure.core.pipeline.policies import HeadersPolicy, ProxyPolicy, \
            RetryPolicy, CustomHookPolicy, RedirectPolicy, SansIOHTTPPolicy
        from azure.cli.core.sdk.policies import SafeNetworkTraceLoggingPolicy, RecordTelemetryUserAgentPolicy
        from ._http_policy import AAZBearerTokenCredentialPolicy

        self.user_agent_policy = kwargs.get('user_agent_policy') or RecordTelemetryUserAgentPolicy(**kwargs)
        self.headers_policy = kwargs.get('headers_policy') or HeadersPolicy(**kwargs)
        self.proxy_policy = kwargs.get('proxy_policy') or ProxyPolicy(**kwargs)
        self.logging_policy = kwargs.get('logging_policy') or SafeNetworkTraceLoggingPolicy(**kwargs)
        self.http_logging_policy = kwargs.get('http_logging_policy') or SansIOHTTPPolicy()
        self.retry_policy = kwargs.get('retry_policy') or RetryPolicy(**kwargs)
        self.custom_hook_policy = kwargs.get('custom_hook_policy') or CustomHookPolicy(**kwargs)
        self.redirect_policy = kwargs.get('redirect_policy') or RedirectPolicy(**kwargs)
        self.authentication_policy = kwargs.get('authentication_policy')
        if self.credential and not self.authentication_policy:
            self.authentication_policy = AAZBearerTokenCredentialPolicy(
                self.credential, *self.credential_scopes, **kwargs)


class AAZBaseClient(PipelineClient):
    """Base Client"""

    def __init__(self, ctx, credential, **kwargs):
        base_url = self._build_base_url(ctx, **kwargs)
        if not base_url:
            raise CloudEndpointNotSetException()
        if not kwargs.get('custom_hook_policy'):
            from azure.cli.core.sdk.policies import get_custom_hook_policy
            kwargs['custom_hook_policy'] = get_custom_hook_policy(ctx.cli_ctx)
        super().__init__(
            base_url=base_url,
            config=self._build_configuration(ctx, credential, **kwargs),
            per_call_policies=self._build_per_call_policies(ctx, **kwargs)
        )

    @classmethod
    @abstractmethod
    def _build_base_url(cls, ctx, **kwargs):
        """Provide a complete url. Supports placeholder added"""
        # return "https://{KeyVaultName}" + ctx.cli_ctx.cloud.suffixes.keyvault_dns
        raise NotImplementedError()

    @classmethod
    @abstractmethod
    def _build_configuration(cls, ctx, credential, **kwargs):
        """Provide client configuration"""
        # return AAZClientConfiguration(
        #     credential=credential,
        #     credential_scopes=<credential_scopes>,
        #     **kwargs
        # )
        raise NotImplementedError()

    @classmethod
    def _build_per_call_policies(cls, ctx, **kwargs):  # pylint: disable=unused-argument
        return []

    def send_request(self, request, stream=False, **kwargs):  # pylint: disable=arguments-differ
        session = self._pipeline.run(request, stream=stream, **kwargs)
        return session

    def build_lro_polling(self, no_wait, initial_session, deserialization_callback, error_callback,
                          lro_options=None, path_format_arguments=None):
        from azure.core.polling.base_polling import OperationResourcePolling
        from ._poller import AAZNoPolling, AAZBasePolling
        if no_wait == True:  # noqa: E712, pylint: disable=singleton-comparison
            polling = AAZNoPolling()
        else:
            polling = AAZBasePolling(
                lro_options=lro_options,
                lro_algorithms=[
                    OperationResourcePolling(lro_options=lro_options),
                    LocationPolling(),
                    StatusCheckPolling(),
                ],
                path_format_arguments=path_format_arguments,
                http_response_error_callback=error_callback,
            )

        polling.initialize(
            self,
            initial_response=initial_session,
            deserialization_callback=deserialization_callback
        )
        return polling

    @classmethod
    def get_cloud_endpoint(cls, ctx, arm_idx):
        from azure.cli.core.cloud import CloudEndpoints
        endpoints = None
        # retrieve by indexes mapping
        for property_name, value in CloudEndpoints.ARM_METADATA_INDEX.items():
            if value == arm_idx:
                try:
                    endpoints = getattr(ctx.cli_ctx.cloud.endpoints, property_name)
                except CloudEndpointNotSetException:
                    break

        # retrieve from arm response
        if not endpoints:
            endpoints = cls._retrieve_value_in_arm_cloud_metadata(ctx, arm_idx)
        return endpoints

    @classmethod
    def get_cloud_suffix(cls, ctx, arm_idx):
        from azure.cli.core.cloud import CloudSuffixes
        # retrieve by indexes mapping
        suffixes = None
        for property_name, value in CloudSuffixes.ARM_METADATA_INDEX.items():
            if value == arm_idx:
                try:
                    suffixes = getattr(ctx.cli_ctx.cloud.suffixes, property_name)
                except CloudSuffixNotSetException:
                    break

        if not suffixes:
            # retrieve from arm response
            suffixes = cls._retrieve_value_in_arm_cloud_metadata(ctx, arm_idx)
        if suffixes and not suffixes.startswith('.'):
            # always add '.' when suffixes is not None
            suffixes = '.' + suffixes
        return suffixes

    @staticmethod
    def _retrieve_value_in_arm_cloud_metadata(ctx, arm_idx):
        from azure.cli.core.cloud import retrieve_arm_cloud_metadata, get_active_cloud_name
        from ._utils import AAZShortHandSyntaxParser, AAZInvalidShorthandSyntaxError

        cloud_name = get_active_cloud_name(ctx.cli_ctx)
        arm_cloud = None
        try:
            arm_cloud_metadata = retrieve_arm_cloud_metadata()
            for cloud in arm_cloud_metadata:
                if cloud.get('name', None) == cloud_name:
                    arm_cloud = cloud
                    break
            if not arm_cloud:
                return None
        except Exception:  # pylint: disable=broad-except
            return None

        try:
            value = arm_cloud
            for key in AAZShortHandSyntaxParser.parse_partial_value_key(arm_idx):
                value = value[key]
        except AAZInvalidShorthandSyntaxError as err:
            logger.warning('Invalid metadata index: %s', err)
            raise err
        except LookupError:
            logger.debug(
                "Failed to retrieve '%s' property in cloud metadata from the url specified by ARM_CLOUD_METADATA_URL",
                arm_idx
            )
            return None
        if not isinstance(value, str):
            logger.debug(
                "Failed to retrieve '%s' property in cloud metadata from the url specified by ARM_CLOUD_METADATA_URL: "
                "property is not a string", arm_idx)
            return None
        return value


@register_client("MgmtClient")
class AAZMgmtClient(AAZBaseClient):
    """Management Client for Management Plane APIs"""

    @classmethod
    def _build_base_url(cls, ctx, **kwargs):
        return ctx.cli_ctx.cloud.endpoints.resource_manager

    @classmethod
    def _build_configuration(cls, ctx, credential, **kwargs):
        from azure.cli.core.auth.util import resource_to_scopes
        return AAZClientConfiguration(
            credential=credential,
            credential_scopes=resource_to_scopes(ctx.cli_ctx.cloud.endpoints.active_directory_resource_id),
            **kwargs
        )

    @classmethod
    def _build_per_call_policies(cls, ctx, **kwargs):
        from azure.mgmt.core.policies import ARMAutoResourceProviderRegistrationPolicy
        return [ARMAutoResourceProviderRegistrationPolicy()]

    def build_lro_polling(self, no_wait, initial_session, deserialization_callback, error_callback,
                          lro_options=None, path_format_arguments=None):
        from azure.mgmt.core.polling.arm_polling import AzureAsyncOperationPolling, BodyContentPolling
        from ._poller import AAZNoPolling, AAZBasePolling
        if no_wait == True:  # noqa: E712, pylint: disable=singleton-comparison
            polling = AAZNoPolling()
        else:
            polling = AAZBasePolling(
                lro_options=lro_options,
                lro_algorithms=[
                    AzureAsyncOperationPolling(lro_options=lro_options),
                    LocationPolling(),
                    BodyContentPolling(),
                    StatusCheckPolling(),
                ],
                path_format_arguments=path_format_arguments,
                http_response_error_callback=error_callback,
            )

        polling.initialize(
            self,
            initial_response=initial_session,
            deserialization_callback=deserialization_callback
        )
        return polling



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "network network-watcher packet-capture create",
)
class Create(AAZCommand):
    """Create and start a packet capture on the specified VM.

    :example: Create and start a packet capture 
        az network network-watcher packet-capture create --network-watcher-name "NetworkWatcher_eastus2euap" --packet-capture-name "clitestpcap" --resource-group "NetworkWatcherRG" --storage-location '{"storageId": "/subscriptions//resourceGroups//providers/Microsoft.Storage/storageAccounts/", "filePath": "C:\Captures\testByCli.cap"}' --target "/subscriptions/*****/resourceGroups//providers/Microsoft.Compute/virtualMachines/testVmName"
    """

    _aaz_info = {
        "version": "2024-05-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.network/networkwatchers/{}/packetcaptures/{}", "2024-05-01"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.network_watcher_name = AAZStrArg(
            options=["--network-watcher-name"],
            help="The name of the network watcher.",
            required=True,
        )
        _args_schema.packet_capture_name = AAZStrArg(
            options=["-n", "--name", "--packet-capture-name"],
            help="The name of the packet capture session.",
            required=True,
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )

        # define Arg Group "Properties"

        _args_schema = cls._args_schema
        _args_schema.bytes_to_capture_per_packet = AAZIntArg(
            options=["--bytes-to-capture", "--bytes-to-capture-per-packet"],
            arg_group="Properties",
            help="Number of bytes captured per packet, the remaining bytes are truncated.",
            default=0,
            fmt=AAZIntArgFormat(
                maximum=4294967295,
                minimum=0,
            ),
        )
        _args_schema.capture_settings = AAZObjectArg(
            options=["--capture-settings"],
            arg_group="Properties",
            help="The capture setting holds the 'FileCount', 'FileSizeInBytes', 'SessionTimeLimitInSeconds' values.",
        )
        _args_schema.continuous_capture = AAZBoolArg(
            options=["--continuous-capture"],
            arg_group="Properties",
            help="This continuous capture is a nullable boolean, which can hold 'null', 'true' or 'false' value. If we do not pass this parameter, it would be consider as 'null', default value is 'null'.",
        )
        _args_schema.filters = AAZListArg(
            options=["--filters"],
            arg_group="Properties",
            help="A list of packet capture filters.",
        )
        _args_schema.scope = AAZObjectArg(
            options=["--scope"],
            arg_group="Properties",
            help="A list of AzureVMSS instances which can be included or excluded to run packet capture. If both included and excluded are empty, then the packet capture will run on all instances of AzureVMSS.",
        )
        _args_schema.storage_location = AAZObjectArg(
            options=["--storage-location"],
            arg_group="Properties",
            help="The storage location for a packet capture session.",
            required=True,
        )
        _args_schema.target = AAZStrArg(
            options=["--target"],
            arg_group="Properties",
            help="The ID of the targeted resource, only AzureVM and AzureVMSS as target type are currently supported.",
            required=True,
        )
        _args_schema.target_type = AAZStrArg(
            options=["--target-type"],
            arg_group="Properties",
            help="Target type of the resource provided.",
            enum={"AzureVM": "AzureVM", "AzureVMSS": "AzureVMSS"},
        )
        _args_schema.time_limit_in_seconds = AAZIntArg(
            options=["--time-limit-in-seconds"],
            arg_group="Properties",
            help="Maximum duration of the capture session in seconds.",
            fmt=AAZIntArgFormat(
                maximum=18000,
                minimum=0,
            ),
        )
        _args_schema.total_bytes_per_session = AAZIntArg(
            options=["--total-bytes", "--total-bytes-per-session"],
            arg_group="Properties",
            help="Maximum size of the capture output.",
            fmt=AAZIntArgFormat(
                maximum=4294967295,
                minimum=0,
            ),
        )

        capture_settings = cls._args_schema.capture_settings
        capture_settings.file_count = AAZIntArg(
            options=["file-count"],
            help="Number of file count. Default value of count is 10 and maximum number is 10000.",
            default=10,
            fmt=AAZIntArgFormat(
                maximum=10000,
                minimum=0,
            ),
        )
        capture_settings.file_size_in_bytes = AAZIntArg(
            options=["file-size-in-bytes"],
            help="Number of bytes captured per packet. Default value in bytes 104857600 (100MB) and maximum in bytes 4294967295 (4GB).",
            default=104857600,
            fmt=AAZIntArgFormat(
                maximum=4294967295,
                minimum=0,
            ),
        )
        capture_settings.session_time_limit_in_seconds = AAZIntArg(
            options=["session-time-limit-in-seconds"],
            help="Maximum duration of the capture session in seconds is 604800s (7 days) for a file. Default value in second 86400s (1 day).",
            default=86400,
            fmt=AAZIntArgFormat(
                maximum=604800,
                minimum=0,
            ),
        )

        filters = cls._args_schema.filters
        filters.Element = AAZObjectArg()

        _element = cls._args_schema.filters.Element
        _element.local_ip_address = AAZStrArg(
            options=["local-ip-address"],
            help="Local IP Address to be filtered on. Notation: \"127.0.0.1\" for single address entry. \"127.0.0.1-127.0.0.255\" for range. \"127.0.0.1;127.0.0.5\"? for multiple entries. Multiple ranges not currently supported. Mixing ranges with multiple entries not currently supported. Default = null.",
        )
        _element.local_port = AAZStrArg(
            options=["local-port"],
            help="Local port to be filtered on. Notation: \"80\" for single port entry.\"80-85\" for range. \"80;443;\" for multiple entries. Multiple ranges not currently supported. Mixing ranges with multiple entries not currently supported. Default = null.",
        )
        _element.protocol = AAZStrArg(
            options=["protocol"],
            help="Protocol to be filtered on.",
            default="Any",
            enum={"Any": "Any", "TCP": "TCP", "UDP": "UDP"},
        )
        _element.remote_ip_address = AAZStrArg(
            options=["remote-ip-address"],
            help="Local IP Address to be filtered on. Notation: \"127.0.0.1\" for single address entry. \"127.0.0.1-127.0.0.255\" for range. \"127.0.0.1;127.0.0.5;\" for multiple entries. Multiple ranges not currently supported. Mixing ranges with multiple entries not currently supported. Default = null.",
        )
        _element.remote_port = AAZStrArg(
            options=["remote-port"],
            help="Remote port to be filtered on. Notation: \"80\" for single port entry.\"80-85\" for range. \"80;443;\" for multiple entries. Multiple ranges not currently supported. Mixing ranges with multiple entries not currently supported. Default = null.",
        )

        scope = cls._args_schema.scope
        scope.exclude = AAZListArg(
            options=["exclude"],
            help="List of AzureVMSS instances which has to be excluded from the AzureVMSS from running packet capture.",
        )
        scope.include = AAZListArg(
            options=["include"],
            help="List of AzureVMSS instances to run packet capture on.",
        )

        exclude = cls._args_schema.scope.exclude
        exclude.Element = AAZStrArg()

        include = cls._args_schema.scope.include
        include.Element = AAZStrArg()

        storage_location = cls._args_schema.storage_location
        storage_location.file_path = AAZStrArg(
            options=["file-path"],
            help="This path is invalid if 'Continuous Capture' is provided with 'true' or 'false'. A valid local path on the targeting VM. Must include the name of the capture file (*.cap). For linux virtual machine it must start with /var/captures. Required if no storage ID is provided, otherwise optional.",
        )
        storage_location.local_path = AAZStrArg(
            options=["local-path"],
            help="This path is valid if 'Continuous Capture' is provided with 'true' or 'false' and required if no storage ID is provided, otherwise optional. Must include the name of the capture file (*.cap). For linux virtual machine it must start with /var/captures.",
        )
        storage_location.storage_id = AAZStrArg(
            options=["storage-id"],
            help="The ID of the storage account to save the packet capture session. Required if no localPath or filePath is provided.",
        )
        storage_location.storage_path = AAZStrArg(
            options=["storage-path"],
            help="The URI of the storage path to save the packet capture. Must be a well-formed URI describing the location to save the packet capture.",
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        yield self.PacketCapturesCreate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class PacketCapturesCreate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [201]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/networkWatchers/{networkWatcherName}/packetCaptures/{packetCaptureName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "networkWatcherName", self.ctx.args.network_watcher_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "packetCaptureName", self.ctx.args.packet_capture_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2024-05-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                typ=AAZObjectType,
                typ_kwargs={"flags": {"required": True, "client_flatten": True}}
            )
            _builder.set_prop("properties", AAZObjectType, ".", typ_kwargs={"flags": {"required": True, "client_flatten": True}})

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("bytesToCapturePerPacket", AAZIntType, ".bytes_to_capture_per_packet")
                properties.set_prop("captureSettings", AAZObjectType, ".capture_settings")
                properties.set_prop("continuousCapture", AAZBoolType, ".continuous_capture")
                properties.set_prop("filters", AAZListType, ".filters")
                properties.set_prop("scope", AAZObjectType, ".scope")
                properties.set_prop("storageLocation", AAZObjectType, ".storage_location", typ_kwargs={"flags": {"required": True}})
                properties.set_prop("target", AAZStrType, ".target", typ_kwargs={"flags": {"required": True}})
                properties.set_prop("targetType", AAZStrType, ".target_type")
                properties.set_prop("timeLimitInSeconds", AAZIntType, ".time_limit_in_seconds")
                properties.set_prop("totalBytesPerSession", AAZIntType, ".total_bytes_per_session")

            capture_settings = _builder.get(".properties.captureSettings")
            if capture_settings is not None:
                capture_settings.set_prop("fileCount", AAZIntType, ".file_count")
                capture_settings.set_prop("fileSizeInBytes", AAZIntType, ".file_size_in_bytes")
                capture_settings.set_prop("sessionTimeLimitInSeconds", AAZIntType, ".session_time_limit_in_seconds")

            filters = _builder.get(".properties.filters")
            if filters is not None:
                filters.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.filters[]")
            if _elements is not None:
                _elements.set_prop("localIPAddress", AAZStrType, ".local_ip_address")
                _elements.set_prop("localPort", AAZStrType, ".local_port")
                _elements.set_prop("protocol", AAZStrType, ".protocol")
                _elements.set_prop("remoteIPAddress", AAZStrType, ".remote_ip_address")
                _elements.set_prop("remotePort", AAZStrType, ".remote_port")

            scope = _builder.get(".properties.scope")
            if scope is not None:
                scope.set_prop("exclude", AAZListType, ".exclude")
                scope.set_prop("include", AAZListType, ".include")

            exclude = _builder.get(".properties.scope.exclude")
            if exclude is not None:
                exclude.set_elements(AAZStrType, ".")

            include = _builder.get(".properties.scope.include")
            if include is not None:
                include.set_elements(AAZStrType, ".")

            storage_location = _builder.get(".properties.storageLocation")
            if storage_location is not None:
                storage_location.set_prop("filePath", AAZStrType, ".file_path")
                storage_location.set_prop("localPath", AAZStrType, ".local_path")
                storage_location.set_prop("storageId", AAZStrType, ".storage_id")
                storage_location.set_prop("storagePath", AAZStrType, ".storage_path")

            return self.serialize_content(_content_value)

        def on_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_201
            )

        _schema_on_201 = None

        @classmethod
        def _build_schema_on_201(cls):
            if cls._schema_on_201 is not None:
                return cls._schema_on_201

            cls._schema_on_201 = AAZObjectType()

            _schema_on_201 = cls._schema_on_201
            _schema_on_201.etag = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_201.id = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_201.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_201.properties = AAZObjectType(
                flags={"client_flatten": True},
            )

            properties = cls._schema_on_201.properties
            properties.bytes_to_capture_per_packet = AAZIntType(
                serialized_name="bytesToCapturePerPacket",
            )
            properties.capture_settings = AAZObjectType(
                serialized_name="captureSettings",
            )
            properties.continuous_capture = AAZBoolType(
                serialized_name="continuousCapture",
            )
            properties.filters = AAZListType()
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.scope = AAZObjectType()
            properties.storage_location = AAZObjectType(
                serialized_name="storageLocation",
                flags={"required": True},
            )
            properties.target = AAZStrType(
                flags={"required": True},
            )
            properties.target_type = AAZStrType(
                serialized_name="targetType",
            )
            properties.time_limit_in_seconds = AAZIntType(
                serialized_name="timeLimitInSeconds",
            )
            properties.total_bytes_per_session = AAZIntType(
                serialized_name="totalBytesPerSession",
            )

            capture_settings = cls._schema_on_201.properties.capture_settings
            capture_settings.file_count = AAZIntType(
                serialized_name="fileCount",
            )
            capture_settings.file_size_in_bytes = AAZIntType(
                serialized_name="fileSizeInBytes",
            )
            capture_settings.session_time_limit_in_seconds = AAZIntType(
                serialized_name="sessionTimeLimitInSeconds",
            )

            filters = cls._schema_on_201.properties.filters
            filters.Element = AAZObjectType()

            _element = cls._schema_on_201.properties.filters.Element
            _element.local_ip_address = AAZStrType(
                serialized_name="localIPAddress",
            )
            _element.local_port = AAZStrType(
                serialized_name="localPort",
            )
            _element.protocol = AAZStrType()
            _element.remote_ip_address = AAZStrType(
                serialized_name="remoteIPAddress",
            )
            _element.remote_port = AAZStrType(
                serialized_name="remotePort",
            )

            scope = cls._schema_on_201.properties.scope
            scope.exclude = AAZListType()
            scope.include = AAZListType()

            exclude = cls._schema_on_201.properties.scope.exclude
            exclude.Element = AAZStrType()

            include = cls._schema_on_201.properties.scope.include
            include.Element = AAZStrType()

            storage_location = cls._schema_on_201.properties.storage_location
            storage_location.file_path = AAZStrType(
                serialized_name="filePath",
            )
            storage_location.local_path = AAZStrType(
                serialized_name="localPath",
            )
            storage_location.storage_id = AAZStrType(
                serialized_name="storageId",
            )
            storage_location.storage_path = AAZStrType(
                serialized_name="storagePath",
            )

            return cls._schema_on_201


class _CreateHelper:
    """Helper class for Create"""


__all__ = ["Create"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import json

try:
    import unittest.mock as mock
except ImportError:
    import mock
import unittest
import uuid

from azure.cli.testsdk.scenario_tests.recording_processors import (
    RecordingProcessor, SubscriptionRecordingProcessor, AccessTokenReplacer
)


class TestRecordingProcessors(unittest.TestCase):
    def test_recording_processor_base_class(self):
        rp = RecordingProcessor()
        request_sample = {'url': 'https://www.bing,com', 'headers': {'beta': ['value_1', 'value_2']}}
        response_sample = {'body': 'something', 'headers': {'charlie': ['value_3']}}
        self.assertIs(request_sample, rp.process_request(request_sample))  # reference equality
        self.assertIs(response_sample, rp.process_response(response_sample))

        rp.replace_header(request_sample, 'beta', 'value_1', 'replaced_1')
        self.assertSequenceEqual(request_sample['headers']['beta'], ['replaced_1', 'value_2'])

        rp.replace_header(request_sample, 'Beta', 'replaced_1', 'replaced_2')  # case insensitive
        self.assertSequenceEqual(request_sample['headers']['beta'], ['replaced_2', 'value_2'])

        rp.replace_header(request_sample, 'alpha', 'replaced_1', 'replaced_2')  # ignore KeyError
        self.assertSequenceEqual(request_sample['headers']['beta'], ['replaced_2', 'value_2'])

        rp.replace_header_fn(request_sample, 'beta', lambda v: 'customized')
        self.assertSequenceEqual(request_sample['headers']['beta'], ['customized', 'customized'])

    def test_access_token_processor(self):
        replaced_subscription_id = 'test_fake_token'
        rp = AccessTokenReplacer(replaced_subscription_id)

        TOKEN_STR = '{"token_type": "Bearer", "resource": "url", "access_token": "real_token"}'
        token_response_sample = {'body': {'string': TOKEN_STR}}

        self.assertEqual(json.loads(rp.process_response(token_response_sample)['body']['string'])['access_token'],
                         replaced_subscription_id)

        no_token_response_sample = {'body': {'string': '{"location": "westus"}'}}
        self.assertDictEqual(rp.process_response(no_token_response_sample), no_token_response_sample)

    @staticmethod
    def _mock_subscription_request_body(mock_sub_id):
        return json.dumps({
            "location": "westus",
            "properties": {
                "ipConfigurations": [
                    {
                        "properties": {
                            "subnet": {"id": "/Subscriptions/{}/resourceGroups/etc".format(mock_sub_id)},
                            "name": "azure-sample-ip-config"
                        }
                    },
                ]
            }
        }).encode()

    def test_subscription_recording_processor_for_request(self):
        replaced_subscription_id = str(uuid.uuid4())
        rp = SubscriptionRecordingProcessor(replaced_subscription_id)

        uri_templates = ['https://management.azure.com/subscriptions/{}/providers/Microsoft.ContainerRegistry/'
                         'checkNameAvailability?api-version=2017-03-01',
                         'https://graph.windows.net/{}/applications?api-version=1.6']

        for template in uri_templates:
            mock_sub_id = str(uuid.uuid4())
            mock_request = mock.Mock()
            mock_request.uri = template.format(mock_sub_id)
            mock_request.body = self._mock_subscription_request_body(mock_sub_id)
            mock_request.headers = {'content-type': 'application/json'}

            rp.process_request(mock_request)
            self.assertEqual(mock_request.uri, template.format(replaced_subscription_id))
            self.assertEqual(mock_request.body,
                             self._mock_subscription_request_body(replaced_subscription_id))

    def test_subscription_recording_processor_for_response(self):
        replaced_subscription_id = str(uuid.uuid4())
        rp = SubscriptionRecordingProcessor(replaced_subscription_id)

        body_templates = ['https://management.azure.com/subscriptions/{}/providers/Microsoft.ContainerRegistry/'
                          'checkNameAvailability?api-version=2017-03-01',
                          'https://graph.Windows.net/{}/applications?api-version=1.6',
                          "{{'scope':'/subscriptions/{}', 'another_data':'/Microsoft.Something'}}",
                          'https://management.azure.com\\/subscriptions\\/{}\\/providers\\/'
                          'Microsoft.ContainerRegistry\\/checkNameAvailability?api-version=2017-03-01']

        location_header_template = 'https://graph.windows.net/{}/directoryObjects/' \
                                   'f604c53a-aa21-44d5-a41f-c1ef0b5304bd/Microsoft.DirectoryServices.Application'

        asyncoperation_header_template = 'https://management.azure.com/subscriptions/{}/resourceGroups/' \
                                         'clitest.rg000001/providers/Microsoft.Sql/servers/clitestserver000002/' \
                                         'databases/cliautomationdb01/azureAsyncOperation/' \
                                         '6ec6196b-fbaa-415f-8c1a-6cb634a96cb2?api-version=2014-04-01-Preview'

        for template in body_templates:
            mock_sub_id = str(uuid.uuid4())
            mock_response = {
                'body': {'string': template.format(mock_sub_id)},
                'headers': {
                    'Location': [location_header_template.format(mock_sub_id)],
                    'azure-asyncoperation': [asyncoperation_header_template.format(mock_sub_id)],
                    'content-type': ['application/json'],
                }
            }
            rp.process_response(mock_response)
            self.assertEqual(mock_response['body']['string'], template.format(replaced_subscription_id))

            self.assertSequenceEqual(mock_response['headers']['Location'],
                                     [location_header_template.format(replaced_subscription_id)])
            self.assertSequenceEqual(mock_response['headers']['azure-asyncoperation'],
                                     [asyncoperation_header_template.format(replaced_subscription_id)])

    def test_recording_processor_skip_body_on_unrecognized_content_type(self):
        location_header_template = 'https://graph.windows.net/{}/directoryObjects/' \
                                   'f604c53a-aa21-44d5-a41f-c1ef0b5304bd/Microsoft.DirectoryServices.Application'
        replaced_subscription_id = str(uuid.uuid4())
        rp = SubscriptionRecordingProcessor(replaced_subscription_id)

        mock_sub_id = str(uuid.uuid4())
        mock_response = {
            'body': {'string': mock_sub_id},
            'headers': {
                'Location': [location_header_template.format(mock_sub_id)],
                'content-type': ['application/foo'],
            }
        }

        # action
        rp.process_response(mock_response)

        # assert
        self.assertEqual(mock_response['body']['string'], mock_sub_id)  # body unchanged
        self.assertEqual(mock_response['headers']['Location'],
                         [location_header_template.format(replaced_subscription_id)])  # header changed



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest
import importlib

# The path contains a reserved keyword 'import', so we need a workaround here
acr_import = importlib.import_module('azure.cli.command_modules.acr.import')


class TestRegionalEndpointUriConversion(unittest.TestCase):

    def test_valid_regional_endpoint_conversion(self):
        """Test conversion of regional endpoint URIs to standard format."""
        login_server_suffix = '.azurecr.io'

        # Valid regional endpoints that should be converted
        test_cases = [
            ('myregistry.westus.geo.azurecr.io', 'myregistry.azurecr.io'),
            ('registry123.eastus2.geo.azurecr.io', 'registry123.azurecr.io'),
            ('prod-registry.centralus.geo.azurecr.io', 'prod-registry.azurecr.io'),
            # Mixed-case: hostnames are case-insensitive, output is normalized to lowercase
            ('MyRegistry.EastUS.Geo.azurecr.io', 'myregistry.azurecr.io'),
            ('MYREGISTRY.WESTUS.GEO.AZURECR.IO', 'myregistry.azurecr.io'),
        ]

        for regional_uri, expected in test_cases:
            result = acr_import._regional_endpoint_uri_to_login_server(regional_uri, login_server_suffix)
            self.assertEqual(result, expected)


    def test_non_regional_endpoint_uris_unchanged(self):
        """Test that non-regional endpoint URIs are returned unchanged."""
        login_server_suffix = '.azurecr.io'

        # URIs that should remain unchanged
        test_cases = [
            'testregistry.azurecr.io',
            'external-registry.com',
            'testregistry.eastus.notgeo.azurecr.io',
        ]

        for uri in test_cases:
            result = acr_import._regional_endpoint_uri_to_login_server(uri, login_server_suffix)
            self.assertEqual(result, uri)

    @staticmethod
    def _match_regional_endpoint(login_server, endpoint, regional_endpoint_host_names):
        """Replicate the matching logic from acr_login for unit testing."""
        login_server_name = login_server.split('.')[0]
        regional_endpoint_prefix = f"{login_server_name}.{endpoint}.geo.".lower()
        return next(
            (url for url in regional_endpoint_host_names
             if url.lower().strip().startswith(regional_endpoint_prefix)), None)

    def test_match_standard_registry(self):
        """Registry without DNL — login_server starts with registry name."""
        login_server = 'myregistry.azurecr.io'
        hosts = [
            'myregistry.eastus.geo.azurecr.io',
            'myregistry.westus.geo.azurecr.io',
        ]
        self.assertEqual(
            self._match_regional_endpoint(login_server, 'eastus', hosts),
            'myregistry.eastus.geo.azurecr.io')
        self.assertEqual(
            self._match_regional_endpoint(login_server, 'westus', hosts),
            'myregistry.westus.geo.azurecr.io')

    def test_match_dnl_registry(self):
        """Registry with DNL suffix — login_server has a hash appended."""
        login_server = 'myregistry-d7ezgzevdwfvc8ht.azurecr.io'
        hosts = [
            'myregistry-d7ezgzevdwfvc8ht.eastus.geo.azurecr.io',
            'myregistry-d7ezgzevdwfvc8ht.westus.geo.azurecr.io',
        ]
        self.assertEqual(
            self._match_regional_endpoint(login_server, 'eastus', hosts),
            'myregistry-d7ezgzevdwfvc8ht.eastus.geo.azurecr.io')
        self.assertEqual(
            self._match_regional_endpoint(login_server, 'westus', hosts),
            'myregistry-d7ezgzevdwfvc8ht.westus.geo.azurecr.io')

    def test_no_match_returns_none(self):
        """Endpoint region not in the host list returns None."""
        login_server = 'myregistry.azurecr.io'
        hosts = ['myregistry.eastus.geo.azurecr.io']
        self.assertIsNone(
            self._match_regional_endpoint(login_server, 'westus', hosts))

    def test_match_case_insensitive(self):
        """Matching is case-insensitive for both endpoint arg and host names."""
        login_server = 'MyRegistry.azurecr.io'
        hosts = ['MyRegistry.EastUS.geo.azurecr.io']
        self.assertEqual(
            self._match_regional_endpoint(login_server, 'eastus', hosts),
            'MyRegistry.EastUS.geo.azurecr.io')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.util import CLIError


def create_resource_parameters(arg_schema, arg_group=None):
    from azure.cli.core.aaz import AAZResourceGroupNameArg, AAZStrArg
    arg_schema.resource_group_name = AAZResourceGroupNameArg(arg_group=arg_group)
    arg_schema.namespace = AAZStrArg(
        options=['--resource-namespace'],
        help="Target resource provider namespace.",
        arg_group=arg_group,
    )
    arg_schema.parent = AAZStrArg(
        options=['--resource-parent'],
        help="Target resource parent path, if applicable.",
        arg_group=arg_group,
    )
    arg_schema.resource_type = AAZStrArg(
        options=['--resource-type'],
        help="Target resource type. Can also accept namespace/type format (Ex: 'Microsoft.Compute/virtualMachines')",
        arg_group=arg_group,
    )


def update_resource_parameters(ctx, alias="resource"):
    args = ctx.args
    from azure.mgmt.core.tools import parse_resource_id, is_valid_resource_id
    from azure.cli.core.aaz import has_value
    name_or_id = args.resource.to_serialized_data()
    usage_error = CLIError('usage error: --{0} ID | --{0} NAME --resource-group NAME '
                           '--{0}-type TYPE [--{0}-parent PARENT] '
                           '[--{0}-namespace NAMESPACE]'.format(alias))
    if not name_or_id:
        raise usage_error
    if is_valid_resource_id(name_or_id):
        if has_value(args.namespace) or has_value(args.parent) or has_value(args.resource_type):
            raise usage_error
        parsed_id = parse_resource_id(name_or_id)
        subscription_id = parsed_id.get('subscription', None)
        if subscription_id:
            # update subscription_id to support cross tenants
            ctx.update_aux_subscriptions(subscription_id)
    else:
        res_ns = args.namespace.to_serialized_data()
        rg = args.resource_group_name.to_serialized_data()
        res_type = args.resource_type.to_serialized_data()
        parent = args.parent.to_serialized_data()
        if has_value(res_type):
            if '/' in res_type:
                res_ns = res_ns or res_type.rsplit('/', 1)[0]
                res_type = res_type.rsplit('/', 1)[1]

        if not res_ns or not rg or not res_type or not name_or_id:
            raise usage_error

        args.resource = f"/subscriptions/{ctx.subscription_id}/resourceGroups/{rg}/providers/{res_ns}/" \
                        f"{parent + '/' if parent else ''}{res_type}/{name_or_id}"


# pylint: disable=unused-argument, line-too-long
def create_diagnostics_settings(client, name, resource_uri,
                                logs=None,
                                metrics=None,
                                event_hub=None,
                                event_hub_rule=None,
                                storage_account=None,
                                workspace=None,
                                export_to_resource_specific=None):
    from azure.mgmt.monitor.models import DiagnosticSettingsResource
    if export_to_resource_specific and workspace is None:
        raise CLIError('usage error: --workspace and --export-to-specific-resource')
    parameters = DiagnosticSettingsResource(storage_account_id=storage_account,
                                            workspace_id=workspace,
                                            event_hub_name=event_hub,
                                            event_hub_authorization_rule_id=event_hub_rule,
                                            metrics=metrics,
                                            logs=logs,
                                            log_analytics_destination_type='Dedicated' if export_to_resource_specific else None)

    return client.create_or_update(resource_uri=resource_uri, parameters=parameters, name=name)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "network express-route gateway update",
)
class Update(AAZCommand):
    """Update settings of an ExpressRoute gateway.

    :example: Update an ExpressRoute gateway.
        az network express-route gateway update --name MyExpressRouteGateway --resource-group MyResourceGroup --min-val 3
    """

    _aaz_info = {
        "version": "2023-09-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.network/expressroutegateways/{}", "2023-09-01"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    AZ_SUPPORT_GENERIC_UPDATE = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.name = AAZStrArg(
            options=["-n", "--name"],
            help="ExpressRoute gateway name.",
            required=True,
            id_part="name",
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        _args_schema.location = AAZResourceLocationArg(
            help="Location. Values from: `az account list-locations`. You can configure the default location using `az configure --defaults location=<location>`.",
            nullable=True,
            fmt=AAZResourceLocationArgFormat(
                resource_group_arg="resource_group",
            ),
        )
        _args_schema.allow_non_vwan_traffic = AAZBoolArg(
            options=["--allow-non-vwan-traffic"],
            help="Configures this gateway to accept traffic from non Virtual WAN networks.",
            nullable=True,
        )
        _args_schema.virtual_hub = AAZStrArg(
            options=["--virtual-hub"],
            help="Name or ID of the virtual hub to associate with the gateway.",
            nullable=True,
        )
        _args_schema.tags = AAZDictArg(
            options=["--tags"],
            help="Space-separated tags: key[=value] [key[=value] ...]. Use \"\" to clear existing tags.",
            nullable=True,
        )

        tags = cls._args_schema.tags
        tags.Element = AAZStrArg(
            nullable=True,
        )

        # define Arg Group "Autoscale"

        _args_schema = cls._args_schema
        _args_schema.max_val = AAZIntArg(
            options=["--max-val"],
            arg_group="Autoscale",
            help="Maximum number of scale units deployed for gateway.",
            nullable=True,
        )
        _args_schema.min_val = AAZIntArg(
            options=["--min-val"],
            arg_group="Autoscale",
            help="Minimum number of scale units deployed for gateway.  Default: 2.",
            nullable=True,
        )

        # define Arg Group "Properties"

        # define Arg Group "PutExpressRouteGatewayParameters"
        return cls._args_schema

    _args_sub_resource_update = None

    @classmethod
    def _build_args_sub_resource_update(cls, _schema):
        if cls._args_sub_resource_update is not None:
            _schema.id = cls._args_sub_resource_update.id
            return

        cls._args_sub_resource_update = AAZObjectArg(
            nullable=True,
        )

        sub_resource_update = cls._args_sub_resource_update
        sub_resource_update.id = AAZStrArg(
            options=["id"],
            help="Resource ID.",
            nullable=True,
        )

        _schema.id = cls._args_sub_resource_update.id

    def _execute_operations(self):
        self.pre_operations()
        self.ExpressRouteGatewaysGet(ctx=self.ctx)()
        self.pre_instance_update(self.ctx.vars.instance)
        self.InstanceUpdateByJson(ctx=self.ctx)()
        self.InstanceUpdateByGeneric(ctx=self.ctx)()
        self.post_instance_update(self.ctx.vars.instance)
        yield self.ExpressRouteGatewaysCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    @register_callback
    def pre_instance_update(self, instance):
        pass

    @register_callback
    def post_instance_update(self, instance):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class ExpressRouteGatewaysGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/expressRouteGateways/{expressRouteGatewayName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "expressRouteGatewayName", self.ctx.args.name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2023-09-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()
            _UpdateHelper._build_schema_express_route_gateway_read(cls._schema_on_200)

            return cls._schema_on_200

    class ExpressRouteGatewaysCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200, 201]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/expressRouteGateways/{expressRouteGatewayName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "expressRouteGatewayName", self.ctx.args.name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2023-09-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=self.ctx.vars.instance,
            )

            return self.serialize_content(_content_value)

        def on_200_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200_201
            )

        _schema_on_200_201 = None

        @classmethod
        def _build_schema_on_200_201(cls):
            if cls._schema_on_200_201 is not None:
                return cls._schema_on_200_201

            cls._schema_on_200_201 = AAZObjectType()
            _UpdateHelper._build_schema_express_route_gateway_read(cls._schema_on_200_201)

            return cls._schema_on_200_201

    class InstanceUpdateByJson(AAZJsonInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance(self.ctx.vars.instance)

        def _update_instance(self, instance):
            _instance_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=instance,
                typ=AAZObjectType
            )
            _builder.set_prop("location", AAZStrType, ".location")
            _builder.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})
            _builder.set_prop("tags", AAZDictType, ".tags")

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("allowNonVirtualWanTraffic", AAZBoolType, ".allow_non_vwan_traffic")
                properties.set_prop("autoScaleConfiguration", AAZObjectType)
                properties.set_prop("virtualHub", AAZObjectType, ".", typ_kwargs={"flags": {"required": True}})

            auto_scale_configuration = _builder.get(".properties.autoScaleConfiguration")
            if auto_scale_configuration is not None:
                auto_scale_configuration.set_prop("bounds", AAZObjectType)

            bounds = _builder.get(".properties.autoScaleConfiguration.bounds")
            if bounds is not None:
                bounds.set_prop("max", AAZIntType, ".max_val")
                bounds.set_prop("min", AAZIntType, ".min_val")

            virtual_hub = _builder.get(".properties.virtualHub")
            if virtual_hub is not None:
                virtual_hub.set_prop("id", AAZStrType, ".virtual_hub")

            tags = _builder.get(".tags")
            if tags is not None:
                tags.set_elements(AAZStrType, ".")

            return _instance_value

    class InstanceUpdateByGeneric(AAZGenericInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance_by_generic(
                self.ctx.vars.instance,
                self.ctx.generic_update_args
            )


class _UpdateHelper:
    """Helper class for Update"""

    @classmethod
    def _build_schema_sub_resource_update(cls, _builder):
        if _builder is None:
            return
        _builder.set_prop("id", AAZStrType, ".id")

    _schema_express_route_gateway_read = None

    @classmethod
    def _build_schema_express_route_gateway_read(cls, _schema):
        if cls._schema_express_route_gateway_read is not None:
            _schema.etag = cls._schema_express_route_gateway_read.etag
            _schema.id = cls._schema_express_route_gateway_read.id
            _schema.location = cls._schema_express_route_gateway_read.location
            _schema.name = cls._schema_express_route_gateway_read.name
            _schema.properties = cls._schema_express_route_gateway_read.properties
            _schema.tags = cls._schema_express_route_gateway_read.tags
            _schema.type = cls._schema_express_route_gateway_read.type
            return

        cls._schema_express_route_gateway_read = _schema_express_route_gateway_read = AAZObjectType()

        express_route_gateway_read = _schema_express_route_gateway_read
        express_route_gateway_read.etag = AAZStrType(
            flags={"read_only": True},
        )
        express_route_gateway_read.id = AAZStrType()
        express_route_gateway_read.location = AAZStrType()
        express_route_gateway_read.name = AAZStrType(
            flags={"read_only": True},
        )
        express_route_gateway_read.properties = AAZObjectType(
            flags={"client_flatten": True},
        )
        express_route_gateway_read.tags = AAZDictType()
        express_route_gateway_read.type = AAZStrType(
            flags={"read_only": True},
        )

        properties = _schema_express_route_gateway_read.properties
        properties.allow_non_virtual_wan_traffic = AAZBoolType(
            serialized_name="allowNonVirtualWanTraffic",
        )
        properties.auto_scale_configuration = AAZObjectType(
            serialized_name="autoScaleConfiguration",
        )
        properties.express_route_connections = AAZListType(
            serialized_name="expressRouteConnections",
        )
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.virtual_hub = AAZObjectType(
            serialized_name="virtualHub",
            flags={"required": True},
        )

        auto_scale_configuration = _schema_express_route_gateway_read.properties.auto_scale_configuration
        auto_scale_configuration.bounds = AAZObjectType()

        bounds = _schema_express_route_gateway_read.properties.auto_scale_configuration.bounds
        bounds.max = AAZIntType()
        bounds.min = AAZIntType()

        express_route_connections = _schema_express_route_gateway_read.properties.express_route_connections
        express_route_connections.Element = AAZObjectType()

        _element = _schema_express_route_gateway_read.properties.express_route_connections.Element
        _element.id = AAZStrType()
        _element.name = AAZStrType(
            flags={"required": True},
        )
        _element.properties = AAZObjectType(
            flags={"client_flatten": True},
        )

        properties = _schema_express_route_gateway_read.properties.express_route_connections.Element.properties
        properties.authorization_key = AAZStrType(
            serialized_name="authorizationKey",
        )
        properties.enable_internet_security = AAZBoolType(
            serialized_name="enableInternetSecurity",
        )
        properties.enable_private_link_fast_path = AAZBoolType(
            serialized_name="enablePrivateLinkFastPath",
        )
        properties.express_route_circuit_peering = AAZObjectType(
            serialized_name="expressRouteCircuitPeering",
            flags={"required": True},
        )
        properties.express_route_gateway_bypass = AAZBoolType(
            serialized_name="expressRouteGatewayBypass",
        )
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.routing_configuration = AAZObjectType(
            serialized_name="routingConfiguration",
        )
        properties.routing_weight = AAZIntType(
            serialized_name="routingWeight",
        )

        express_route_circuit_peering = _schema_express_route_gateway_read.properties.express_route_connections.Element.properties.express_route_circuit_peering
        express_route_circuit_peering.id = AAZStrType()

        routing_configuration = _schema_express_route_gateway_read.properties.express_route_connections.Element.properties.routing_configuration
        routing_configuration.associated_route_table = AAZObjectType(
            serialized_name="associatedRouteTable",
        )
        cls._build_schema_sub_resource_read(routing_configuration.associated_route_table)
        routing_configuration.inbound_route_map = AAZObjectType(
            serialized_name="inboundRouteMap",
        )
        cls._build_schema_sub_resource_read(routing_configuration.inbound_route_map)
        routing_configuration.outbound_route_map = AAZObjectType(
            serialized_name="outboundRouteMap",
        )
        cls._build_schema_sub_resource_read(routing_configuration.outbound_route_map)
        routing_configuration.propagated_route_tables = AAZObjectType(
            serialized_name="propagatedRouteTables",
        )
        routing_configuration.vnet_routes = AAZObjectType(
            serialized_name="vnetRoutes",
        )

        propagated_route_tables = _schema_express_route_gateway_read.properties.express_route_connections.Element.properties.routing_configuration.propagated_route_tables
        propagated_route_tables.ids = AAZListType()
        propagated_route_tables.labels = AAZListType()

        ids = _schema_express_route_gateway_read.properties.express_route_connections.Element.properties.routing_configuration.propagated_route_tables.ids
        ids.Element = AAZObjectType()
        cls._build_schema_sub_resource_read(ids.Element)

        labels = _schema_express_route_gateway_read.properties.express_route_connections.Element.properties.routing_configuration.propagated_route_tables.labels
        labels.Element = AAZStrType()

        vnet_routes = _schema_express_route_gateway_read.properties.express_route_connections.Element.properties.routing_configuration.vnet_routes
        vnet_routes.bgp_connections = AAZListType(
            serialized_name="bgpConnections",
            flags={"read_only": True},
        )
        vnet_routes.static_routes = AAZListType(
            serialized_name="staticRoutes",
        )
        vnet_routes.static_routes_config = AAZObjectType(
            serialized_name="staticRoutesConfig",
        )

        bgp_connections = _schema_express_route_gateway_read.properties.express_route_connections.Element.properties.routing_configuration.vnet_routes.bgp_connections
        bgp_connections.Element = AAZObjectType()
        cls._build_schema_sub_resource_read(bgp_connections.Element)

        static_routes = _schema_express_route_gateway_read.properties.express_route_connections.Element.properties.routing_configuration.vnet_routes.static_routes
        static_routes.Element = AAZObjectType()

        _element = _schema_express_route_gateway_read.properties.express_route_connections.Element.properties.routing_configuration.vnet_routes.static_routes.Element
        _element.address_prefixes = AAZListType(
            serialized_name="addressPrefixes",
        )
        _element.name = AAZStrType()
        _element.next_hop_ip_address = AAZStrType(
            serialized_name="nextHopIpAddress",
        )

        address_prefixes = _schema_express_route_gateway_read.properties.express_route_connections.Element.properties.routing_configuration.vnet_routes.static_routes.Element.address_prefixes
        address_prefixes.Element = AAZStrType()

        static_routes_config = _schema_express_route_gateway_read.properties.express_route_connections.Element.properties.routing_configuration.vnet_routes.static_routes_config
        static_routes_config.propagate_static_routes = AAZBoolType(
            serialized_name="propagateStaticRoutes",
            flags={"read_only": True},
        )
        static_routes_config.vnet_local_route_override_criteria = AAZStrType(
            serialized_name="vnetLocalRouteOverrideCriteria",
        )

        virtual_hub = _schema_express_route_gateway_read.properties.virtual_hub
        virtual_hub.id = AAZStrType()

        tags = _schema_express_route_gateway_read.tags
        tags.Element = AAZStrType()

        _schema.etag = cls._schema_express_route_gateway_read.etag
        _schema.id = cls._schema_express_route_gateway_read.id
        _schema.location = cls._schema_express_route_gateway_read.location
        _schema.name = cls._schema_express_route_gateway_read.name
        _schema.properties = cls._schema_express_route_gateway_read.properties
        _schema.tags = cls._schema_express_route_gateway_read.tags
        _schema.type = cls._schema_express_route_gateway_read.type

    _schema_sub_resource_read = None

    @classmethod
    def _build_schema_sub_resource_read(cls, _schema):
        if cls._schema_sub_resource_read is not None:
            _schema.id = cls._schema_sub_resource_read.id
            return

        cls._schema_sub_resource_read = _schema_sub_resource_read = AAZObjectType()

        sub_resource_read = _schema_sub_resource_read
        sub_resource_read.id = AAZStrType()

        _schema.id = cls._schema_sub_resource_read.id


__all__ = ["Update"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import os

DOTNET_TARGET_FRAMEWORK_REGEX = r"^net\d+\.\d+$"
NETCORE_RUNTIME_NAME = "dotnetcore"
ASPDOTNET_RUNTIME_NAME = "aspnet"
DOTNET_RUNTIME_NAME = "dotnet"
NODE_RUNTIME_NAME = "node"
PYTHON_RUNTIME_NAME = "python"
OS_DEFAULT = "Windows"
LINUX_OS_NAME = "linux"
WINDOWS_OS_NAME = "windows"
STATIC_RUNTIME_NAME = "static"  # not an official supported runtime but used for CLI logic
LINUX_SKU_DEFAULT = "P1V2"
FUNCTIONS_VERSIONS = ['4']
LOGICAPPS_NODE_RUNTIME_VERSIONS = ['~14', '~16', '~18']
FUNCTIONS_LINUX_RUNTIME_VERSION_REGEX = r"^.*\|(.*)$"
FUNCTIONS_WINDOWS_RUNTIME_VERSION_REGEX = r"^~(.*)$"
FUNCTIONS_NO_V2_REGIONS = {
    "USNat West",
    "USNat East",
    "USSec West",
    "USSec East"
}
GITHUB_OAUTH_CLIENT_ID = "8d8e1f6000648c575489"
GITHUB_OAUTH_SCOPES = [
    "admin:repo_hook",
    "repo",
    "workflow"
]
LOGICAPP_KIND = "workflowapp"
FUNCTIONAPP_KIND = "functionapp"
LINUXAPP_KIND = "linux"
DOTNET_REFERENCES_DIR_IN_ZIP = ".az-references"


class FUNCTIONS_STACKS_API_KEYS:
    # pylint:disable=too-few-public-methods,too-many-instance-attributes
    def __init__(self):
        self.NAME = 'name'
        self.VALUE = 'value'
        self.DISPLAY = 'display'
        self.PROPERTIES = 'properties'
        self.MAJOR_VERSIONS = 'majorVersions'
        self.DISPLAY_VERSION = 'displayVersion'
        self.RUNTIME_VERSION = 'runtimeVersion'
        self.IS_HIDDEN = 'isHidden'
        self.IS_PREVIEW = 'isPreview'
        self.IS_DEPRECATED = 'isDeprecated'
        self.IS_DEFAULT = 'isDefault'
        self.SITE_CONFIG_DICT = 'siteConfigPropertiesDictionary'
        self.APP_SETTINGS_DICT = 'appSettingsDictionary'
        self.LINUX_FX_VERSION = 'linuxFxVersion'
        self.APPLICATION_INSIGHTS = 'applicationInsights'
        self.SUPPORTED_EXTENSION_VERSIONS = 'supportedFunctionsExtensionVersions'
        self.USE_32_BIT_WORKER_PROC = 'use32BitWorkerProcess'
        self.FUNCTIONS_WORKER_RUNTIME = 'FUNCTIONS_WORKER_RUNTIME'
        self.GIT_HUB_ACTION_SETTINGS = 'git_hub_action_settings'
        self.END_OF_LIFE_DATE = 'endOfLifeDate'


GENERATE_RANDOM_APP_NAMES = os.path.abspath(os.path.join(os.path.abspath(__file__),
                                                         '../resources/GenerateRandomAppNames.json'))

PUBLIC_CLOUD = "AzureCloud"

VERSION_2022_09_01 = "2022-09-01"

LINUX_GITHUB_ACTIONS_WORKFLOW_TEMPLATE_PATH = {
    'node': 'AppService/linux/nodejs-webapp-on-azure.yml',
    'python': 'AppService/linux/python-webapp-on-azure.yml',
    'dotnetcore': 'AppService/linux/aspnet-core-webapp-on-azure.yml',
    'java': 'AppService/linux/java-jar-webapp-on-azure.yml',
    'tomcat': 'AppService/linux/java-war-webapp-on-azure.yml'
}

WINDOWS_GITHUB_ACTIONS_WORKFLOW_TEMPLATE_PATH = {
    'node': 'AppService/windows/nodejs-webapp-on-azure.yml',
    'python': 'AppService/windows/python-webapp-on-azure.yml',
    'dotnetcore': 'AppService/windows/aspnet-core-webapp-on-azure.yml',
    'java': 'AppService/windows/java-jar-webapp-on-azure.yml',
    'tomcat': 'AppService/windows/java-war-webapp-on-azure.yml'
}

LINUX_FUNCTIONAPP_GITHUB_ACTIONS_WORKFLOW_TEMPLATE_PATH = {
    'node': 'FunctionApp/linux-node.js-functionapp-on-azure.yml',
    'python': 'FunctionApp/linux-python-functionapp-on-azure.yml',
    'dotnet': 'FunctionApp/linux-dotnet-functionapp-on-azure.yml',
    'java': 'FunctionApp/linux-java-functionapp-on-azure.yml',
    'powershell': 'FunctionApp/linux-powershell-functionapp-on-azure.yml',
}

WINDOWS_FUNCTIONAPP_GITHUB_ACTIONS_WORKFLOW_TEMPLATE_PATH = {
    'node': 'FunctionApp/windows-node.js-functionapp-on-azure.yml',
    'dotnet': 'FunctionApp/windows-dotnet-functionapp-on-azure.yml',
    'java': 'FunctionApp/windows-java-functionapp-on-azure.yml',
    'powershell': 'FunctionApp/windows-powershell-functionapp-on-azure.yml',
}

DEFAULT_CENTAURI_IMAGE = 'mcr.microsoft.com/azure-functions/dotnet8-quickstart-demo:1.0'
ACR_IMAGE_SUFFIX = ".azurecr.io"

RUNTIME_STATUS_TEXT_MAP = {
    "BuildRequestReceived": "Received build request...",
    "BuildInProgress": "Building the app...",
    "BuildSuccessful": "Build successful.",
    "BuildFailed": "Build failed.",
    "RuntimeStarting": "Starting the site...",
    "RuntimeSuccessful": "Site started successfully.",
    "RuntimeFailed": "Site failed to start."
}

LANGUAGE_EOL_DEPRECATION_NOTICES = {
    "dotnet|3.1": "https://azure.microsoft.com/en-us/updates/extended-support-for-"
                  "microsoft-net-core-31-will-end-on-3-december-2022/",
    "dotnet|6": "https://azure.microsoft.com/en-us/updates/dotnet6support/",
    "dotnet-isolated|6": "https://azure.microsoft.com/en-us/updates/dotnet6support/",
    "dotnet-isolated|7": "https://azure.microsoft.com/en-us/updates/dotnet7support/",
    "python|3.6": "https://azure.microsoft.com/en-us/updates/azure-functions-support-"
                  "for-python-36-is-ending-on-30-september-2022/",
    "python|3.7": "https://azure.microsoft.com/en-us/updates/community-support-for-"
                  "python-37-is-ending-on-27-june-2023/",
    "python|3.8": "https://azure.microsoft.com/en-us/updates/azure-functions-support-"
                  "for-python-38-is-ending-on-14-october-2023-6/",
    "powershell|6.2": "https://azure.microsoft.com/en-us/updates/azure-functions-support-"
                      "for-powershell-6-is-ending-on-30-september-2022/",
    "node|6": "https://azure.microsoft.com/en-us/updates/azure-functions-support-for-"
              "node-6-is-ending-on-28-february-2022/",
    "node|8": "https://azure.microsoft.com/en-us/updates/azure-functions-support-for-"
              "node-8-is-ending-on-28-february-2022/",
    "node|10": "https://azure.microsoft.com/en-us/updates/azure-functions-support-"
               "for-node-10-is-ending-on-30-september-2022/",
    "node|12": "https://azure.microsoft.com/en-us/updates/node12/",
    "node|14": "https://azure.microsoft.com/en-us/updates/community-support-for-node-"
               "14-lts-is-ending-on-30-april-2023/",
    "node|16": "https://azure.microsoft.com/en-us/updates/node16support/"
}

FLEX_SUBNET_DELEGATION = "Microsoft.App/environments"

DEPLOYMENT_STORAGE_AUTH_TYPES = ['SystemAssignedIdentity', 'UserAssignedIdentity', 'StorageAccountConnectionString']

UPDATE_STRATEGY_TYPES = ['Recreate', 'RollingUpdate']

STORAGE_BLOB_DATA_CONTRIBUTOR_ROLE_ID = 'ba92f5b4-2d11-453d-a403-e96b0029c9fe'



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# AZURE CLI EventHub - NAMESPACE TEST DEFINITIONS

import time

from azure.cli.testsdk import (ScenarioTest, ResourceGroupPreparer, KeyVaultPreparer)


# pylint: disable=line-too-long
# pylint: disable=too-many-lines


class EHNamespaceMSITesting(ScenarioTest):
    from azure.cli.testsdk.scenario_tests import AllowLargeResponse

    @AllowLargeResponse()
    @ResourceGroupPreparer(name_prefix='cli_test_eh_namespace')
    def test_eh_namespace_encryption(self, resource_group):
        self.kwargs.update({
            'loc': 'northeurope',
            'rg': resource_group,
            'namespacename': self.create_random_name(prefix='eventhubs-nscli', length=20),
            'namespacename1': self.create_random_name(prefix='eventhubs-nscli', length=20),
            'namespacename2': self.create_random_name(prefix='eventhubs-nscli', length=20),
            'namespacename3': self.create_random_name(prefix='eventhubs-nscli', length=20),
            'namespacename4': self.create_random_name(prefix='eventhubs-nscli', length=20),
            'namespacename6': self.create_random_name(prefix='eventhubs-nscli', length=20),
            'identity1': self.create_random_name(prefix='eh-identity1', length=20),
            'identity2': self.create_random_name(prefix='eh-identity2', length=20),
            'identity3': self.create_random_name(prefix='eh-identity3', length=20),
            'identity4': self.create_random_name(prefix='eh-identity4', length=20),
            'sku': 'Premium',
            'tier': 'Premium',
            'system': 'SystemAssigned',
            'user': 'UserAssigned',
            'systemuser': 'SystemAssigned, UserAssigned',
            'none': 'None',
            'key1': 'key1',
            'key2': 'key2',
            'key3': 'key3',
            'key4': 'key4',
            'key5': 'key5',
            'kv_name': self.create_random_name(prefix='eventhubs-kg', length=20)
        })

        identity1 = self.cmd('identity create --name {identity1} --resource-group {rg}').get_output_in_json()
        self.assertEqual(identity1['name'], self.kwargs['identity1'])
        self.kwargs.update({'id1':identity1['id']})

        identity2 = self.cmd('identity create --name {identity2} --resource-group {rg}').get_output_in_json()
        self.kwargs.update({'id2': identity2['id']})

        identity3 = self.cmd('identity create --name {identity3} --resource-group {rg}').get_output_in_json()
        self.kwargs.update({'id3': identity3['id']})

        identity4 = self.cmd('identity create --name {identity4} --resource-group {rg}').get_output_in_json()
        self.kwargs.update({'id4': identity4['id']})

        keyvaultcreate = self.cmd('keyvault create --resource-group {rg} --name {kv_name} --location {loc} --enable-purge-protection --enable-rbac-authorization false --retention-days 7').get_output_in_json()

        self.kwargs.update({
            'id1object': identity1['principalId'],
            'id2object': identity2['principalId'],
            'id3object': identity3['principalId'],
            'id4object': identity4['principalId'],
            'key_uri': keyvaultcreate['properties']['vaultUri']
        })


        self.cmd(
            'keyvault set-policy -n {kv_name} -g {rg} --object-id {id1object} --key-permissions  all --secret-permission all')
        self.cmd(
            'keyvault set-policy -n {kv_name} -g {rg} --object-id {id2object} --key-permissions  all --secret-permission all')
        self.cmd(
            'keyvault set-policy -n {kv_name} -g {rg} --object-id {id3object} --key-permissions  all --secret-permission all')
        self.cmd(
            'keyvault set-policy -n {kv_name} -g {rg} --object-id {id4object} --key-permissions  all --secret-permission all')
        self.cmd('keyvault key create -n {key1} --vault-name {kv_name}')
        self.cmd('keyvault key create -n {key2} --vault-name {kv_name}')
        self.cmd('keyvault key create -n {key3} --vault-name {kv_name}')

        namespace = self.cmd(
            'eventhubs namespace create --resource-group {rg} --name {namespacename} --sku {sku} --location {loc} --mi-system-assigned --mi-user-assigned {id1} {id2}' +
            ' --encryption-config key-name={key1} key-vault-uri={key_uri} user-assigned-identity={id1}' +
            ' --encryption-config key-name={key2} key-vault-uri={key_uri} user-assigned-identity={id1}' +
            ' --encryption-config key-name={key3} key-vault-uri={key_uri} user-assigned-identity={id1}'
        ).get_output_in_json()

        self.assertEqual(namespace['identity']['type'], self.kwargs['systemuser'])
        n = [i for i in namespace['identity']['userAssignedIdentities']]
        assert len(n) == 2
        n = [i for i in namespace['encryption']['keyVaultProperties']]
        assert len(n) == 3
        self.assertEqual(False, namespace['encryption']['requireInfrastructureEncryption'])

        namespace = self.cmd('eventhubs namespace encryption remove --resource-group {rg} --namespace-name {namespacename}' +
                             ' --encryption-config key-name={key1} key-vault-uri={key_uri} user-assigned-identity={id1}').get_output_in_json()

        self.assertEqual(namespace['identity']['type'], self.kwargs['systemuser'])
        n = [i for i in namespace['identity']['userAssignedIdentities']]
        assert len(n) == 2
        n = [i for i in namespace['encryption']['keyVaultProperties']]
        assert len(n) == 2
        self.assertEqual(False, namespace['encryption']['requireInfrastructureEncryption'])

        namespace = self.cmd('eventhubs namespace encryption add --resource-group {rg} --namespace-name {namespacename}' +
                             ' --encryption-config key-name={key1} key-vault-uri={key_uri} user-assigned-identity={id1}').get_output_in_json()

        self.assertEqual(namespace['identity']['type'], self.kwargs['systemuser'])
        n = [i for i in namespace['identity']['userAssignedIdentities']]
        assert len(n) == 2
        n = [i for i in namespace['encryption']['keyVaultProperties']]
        assert len(n) == 3
        self.assertEqual(False, namespace['encryption']['requireInfrastructureEncryption'])

        namespace = self.cmd('eventhubs namespace encryption remove --resource-group {rg} --namespace-name {namespacename}' +
                             ' --encryption-config key-name={key2} key-vault-uri={key_uri} user-assigned-identity={id1}' +
                             ' --encryption-config key-name={key3} key-vault-uri={key_uri} user-assigned-identity={id1}').get_output_in_json()

        self.assertEqual(namespace['identity']['type'], self.kwargs['systemuser'])
        n = [i for i in namespace['identity']['userAssignedIdentities']]
        assert len(n) == 2
        n = [i for i in namespace['encryption']['keyVaultProperties']]
        assert len(n) == 1
        self.assertEqual(False, namespace['encryption']['requireInfrastructureEncryption'])

        namespace = self.cmd('eventhubs namespace encryption add --resource-group {rg} --namespace-name {namespacename}' +
                             ' --encryption-config key-name={key2} key-vault-uri={key_uri} user-assigned-identity={id1}' +
                             ' --encryption-config key-name={key3} key-vault-uri={key_uri} user-assigned-identity={id1}').get_output_in_json()

        self.assertEqual(namespace['identity']['type'], self.kwargs['systemuser'])
        n = [i for i in namespace['identity']['userAssignedIdentities']]
        assert len(n) == 2
        n = [i for i in namespace['encryption']['keyVaultProperties']]
        assert len(n) == 3
        self.assertEqual(False, namespace['encryption']['requireInfrastructureEncryption'])


        namespace = self.cmd(
            'eventhubs namespace create --resource-group {rg} --name {namespacename2} --sku {sku} --location {loc} --mi-system-assigned --mi-user-assigned {id1} {id2}' +
            ' --encryption-config key-name={key1} key-vault-uri={key_uri} user-assigned-identity={id1}' +
            ' --encryption-config key-name={key2} key-vault-uri={key_uri} user-assigned-identity={id1}' +
            ' --encryption-config key-name={key3} key-vault-uri={key_uri} user-assigned-identity={id1}' +
            ' --infra-encryption'
        ).get_output_in_json()

        self.assertEqual(namespace['identity']['type'], self.kwargs['systemuser'])
        n = [i for i in namespace['identity']['userAssignedIdentities']]
        assert len(n) == 2
        n = [i for i in namespace['encryption']['keyVaultProperties']]
        assert len(n) == 3
        self.assertEqual(True, namespace['encryption']['requireInfrastructureEncryption'])

        namespace = self.cmd(
            'eventhubs namespace encryption remove --resource-group {rg} --namespace-name {namespacename2}' +
            ' --encryption-config key-name={key1} key-vault-uri={key_uri} user-assigned-identity={id1}').get_output_in_json()

        self.assertEqual(namespace['identity']['type'], self.kwargs['systemuser'])
        n = [i for i in namespace['identity']['userAssignedIdentities']]
        assert len(n) == 2
        n = [i for i in namespace['encryption']['keyVaultProperties']]
        assert len(n) == 2
        self.assertEqual(True, namespace['encryption']['requireInfrastructureEncryption'])


        namespace = self.cmd(
            'eventhubs namespace encryption add --resource-group {rg} --namespace-name {namespacename2}' +
            ' --encryption-config key-name={key1} key-vault-uri={key_uri} user-assigned-identity={id1}').get_output_in_json()

        self.assertEqual(namespace['identity']['type'], self.kwargs['systemuser'])
        n = [i for i in namespace['identity']['userAssignedIdentities']]
        assert len(n) == 2
        n = [i for i in namespace['encryption']['keyVaultProperties']]
        assert len(n) == 3
        self.assertEqual(True, namespace['encryption']['requireInfrastructureEncryption'])
        namespace = self.cmd(
            'eventhubs namespace create --resource-group {rg} --name {namespacename3} --sku {sku} --location {loc} --mi-system-assigned --mi-user-assigned {id1} {id2}'
        ).get_output_in_json()

        self.assertEqual(namespace['identity']['type'], self.kwargs['systemuser'])
        n = [i for i in namespace['identity']['userAssignedIdentities']]
        assert len(n) == 2

        namespace = self.cmd(
            'eventhubs namespace encryption add --resource-group {rg} --namespace-name {namespacename3}' +
            ' --encryption-config key-name={key1} key-vault-uri={key_uri} user-assigned-identity={id1} --infra-encryption').get_output_in_json()

        self.assertEqual(namespace['identity']['type'], self.kwargs['systemuser'])
        n = [i for i in namespace['identity']['userAssignedIdentities']]
        assert len(n) == 2
        n = [i for i in namespace['encryption']['keyVaultProperties']]
        assert len(n) == 1
        self.assertEqual(True, namespace['encryption']['requireInfrastructureEncryption'])

        namespace = self.cmd(
            'eventhubs namespace encryption add --resource-group {rg} --namespace-name {namespacename3}' +
            ' --encryption-config key-name={key2} key-vault-uri={key_uri} user-assigned-identity={id1}').get_output_in_json()

        self.assertEqual(namespace['identity']['type'], self.kwargs['systemuser'])
        n = [i for i in namespace['identity']['userAssignedIdentities']]
        assert len(n) == 2
        n = [i for i in namespace['encryption']['keyVaultProperties']]
        assert len(n) == 2
        self.assertEqual(True, namespace['encryption']['requireInfrastructureEncryption'])

        namespace = self.cmd(
            'eventhubs namespace encryption remove --resource-group {rg} --namespace-name {namespacename3}' +
            ' --encryption-config key-name={key2} key-vault-uri={key_uri} user-assigned-identity={id1}').get_output_in_json()

        self.assertEqual(namespace['identity']['type'], self.kwargs['systemuser'])
        n = [i for i in namespace['identity']['userAssignedIdentities']]
        assert len(n) == 2
        n = [i for i in namespace['encryption']['keyVaultProperties']]
        assert len(n) == 1
        self.assertEqual(True, namespace['encryption']['requireInfrastructureEncryption'])

        namespace = self.cmd(
            'eventhubs namespace create --resource-group {rg} --name {namespacename4} --sku {sku} --location {loc} --mi-system-assigned'
        ).get_output_in_json()

        self.kwargs.update({'pId': namespace['identity']['principalId']})

        self.cmd(
            'keyvault set-policy -n {kv_name} -g {rg} --object-id {pId} --key-permissions  all')
        namespace = self.cmd(
            'eventhubs namespace encryption add --resource-group {rg} --namespace-name {namespacename4}' +
            ' --encryption-config key-name={key1} key-vault-uri={key_uri}'
            ' --encryption-config key-name={key2} key-vault-uri={key_uri} ').get_output_in_json()
        self.assertEqual(namespace['identity']['type'], self.kwargs['system'])
        n = [i for i in namespace['encryption']['keyVaultProperties']]
        assert len(n) == 2

        self.cmd('eventhubs namespace delete --resource-group {rg} --name {namespacename}')
        self.cmd('eventhubs namespace delete --resource-group {rg} --name {namespacename2}')
        self.cmd('eventhubs namespace delete --resource-group {rg} --name {namespacename3}')
        self.cmd('eventhubs namespace delete --resource-group {rg} --name {namespacename4}')
